@@ -14,87 +14,137 @@
 #include <stdarg.h>
 #include <stdio.h>
 
+using namespace Qt::StringLiterals;
+
 /*
  * To regenerate:
  *  curl -O https://www.iana.org/assignments/cbor-tags/cbor-tags.xml
- *  xsltproc tag-transform.xslt cbor-tags.xml
+ *  ./cbortag.py cbor-tags.xml
+ *
+ * The XHTML URL mentioned in the comment below is a human-readable version of
+ * the same resource.
  */
 
 // GENERATED CODE
 struct CborTagDescription
 {
     QCborTag tag;
-    const char *description;    // with space and parentheses
+    const char *description; // with space and parentheses
 };
 
-// CBOR Tags
+// Concise Binary Object Representation (CBOR) Tags
 static const CborTagDescription tagDescriptions[] = {
     // from https://www.iana.org/assignments/cbor-tags/cbor-tags.xhtml
-    { QCborTag(0), " (Standard date/time string; see Section 2.4.1 [RFC7049])" },
-    { QCborTag(1), " (Epoch-based date/time; see Section 2.4.1 [RFC7049])" },
-    { QCborTag(2), " (Positive bignum; see Section 2.4.2 [RFC7049])" },
-    { QCborTag(3), " (Negative bignum; see Section 2.4.2 [RFC7049])" },
-    { QCborTag(4), " (Decimal fraction; see Section 2.4.3 [RFC7049])" },
-    { QCborTag(5), " (Bigfloat; see Section 2.4.3 [RFC7049])" },
-    { QCborTag(16), " (COSE Single Recipient Encrypted Data Object [RFC8152])" },
-    { QCborTag(17), " (COSE Mac w/o Recipients Object [RFC8152])" },
-    { QCborTag(18), " (COSE Single Signer Data Object [RFC8152])" },
-    { QCborTag(21), " (Expected conversion to base64url encoding; see Section 2.4.4.2 [RFC7049])" },
-    { QCborTag(22), " (Expected conversion to base64 encoding; see Section 2.4.4.2 [RFC7049])" },
-    { QCborTag(23), " (Expected conversion to base16 encoding; see Section 2.4.4.2 [RFC7049])" },
-    { QCborTag(24), " (Encoded CBOR data item; see Section 2.4.4.1 [RFC7049])" },
+    { QCborTag(0), " (Standard date/time string; see Section 3.4.1 [RFC8949])" },
+    { QCborTag(1), " (Epoch-based date/time; see Section 3.4.2 [RFC8949])" },
+    { QCborTag(2), " (Positive bignum; see Section 3.4.3 [RFC8949])" },
+    { QCborTag(3), " (Negative bignum; see Section 3.4.3 [RFC8949])" },
+    { QCborTag(4), " (Decimal fraction; see Section 3.4.4 [RFC8949])" },
+    { QCborTag(5), " (Bigfloat; see Section 3.4.4 [RFC8949])" },
+    { QCborTag(16), " (COSE Single Recipient Encrypted Data Object [RFC9052])" },
+    { QCborTag(17), " (COSE Mac w/o Recipients Object [RFC9052])" },
+    { QCborTag(18), " (COSE Single Signer Data Object [RFC9052])" },
+    { QCborTag(19), " (COSE standalone V2 countersignature [RFC9338])" },
+    { QCborTag(21),
+      " (Expected conversion to base64url encoding; see Section 3.4.5.2 [RFC8949])" },
+    { QCborTag(22), " (Expected conversion to base64 encoding; see Section 3.4.5.2 [RFC8949])" },
+    { QCborTag(23), " (Expected conversion to base16 encoding; see Section 3.4.5.2 [RFC8949])" },
+    { QCborTag(24), " (Encoded CBOR data item; see Section 3.4.5.1 [RFC8949])" },
     { QCborTag(25), " (reference the nth previously seen string)" },
     { QCborTag(26), " (Serialised Perl object with classname and constructor arguments)" },
-    { QCborTag(27), " (Serialised language-independent object with type name and constructor arguments)" },
+    { QCborTag(27),
+      " (Serialised language-independent object with type name and constructor arguments)" },
     { QCborTag(28), " (mark value as (potentially) shared)" },
     { QCborTag(29), " (reference nth marked value)" },
     { QCborTag(30), " (Rational number)" },
-    { QCborTag(32), " (URI; see Section 2.4.4.3 [RFC7049])" },
-    { QCborTag(33), " (base64url; see Section 2.4.4.3 [RFC7049])" },
-    { QCborTag(34), " (base64; see Section 2.4.4.3 [RFC7049])" },
+    { QCborTag(31), " (Absent value in a CBOR Array)" },
+    { QCborTag(32), " (URI; see Section 3.4.5.3 [RFC8949])" },
+    { QCborTag(33), " (base64url; see Section 3.4.5.3 [RFC8949])" },
+    { QCborTag(34), " (base64; see Section 3.4.5.3 [RFC8949])" },
     { QCborTag(35), " (Regular expression; see Section 2.4.4.3 [RFC7049])" },
-    { QCborTag(36), " (MIME message; see Section 2.4.4.3 [RFC7049])" },
-    { QCborTag(37), " (Binary UUID ( section 4.1.2))" },
-    { QCborTag(38), " (Language-tagged string)" },
+    { QCborTag(36), " (MIME message; see Section 3.4.5.3 [RFC8949])" },
+    { QCborTag(37), " (Binary UUID [RFC4122, Section 4.1.2])" },
+    { QCborTag(38), " (Language-tagged string [RFC9290, Appendix A])" },
     { QCborTag(39), " (Identifier)" },
-    { QCborTag(61), " (CBOR Web Token (CWT))" },
-    { QCborTag(96), " (COSE Encrypted Data Object [RFC8152])" },
-    { QCborTag(97), " (COSE MACed Data Object [RFC8152])" },
-    { QCborTag(98), " (COSE Signed Data Object [RFC8152])" },
-    { QCborTag(256), " (mark value as having string references)" },
-    { QCborTag(257), " (Binary MIME message)" },
-    { QCborTag(258), " (Mathematical finite set)" },
-    { QCborTag(260), " (Network Address (IPv4 or IPv6 or MAC Address))" },
-    { QCborTag(264), " (Decimal fraction with arbitrary exponent)" },
-    { QCborTag(265), " (Bigfloat with arbitrary exponent)" },
-    { QCborTag(1001), " (extended time)" },
-    { QCborTag(1002), " (duration)" },
-    { QCborTag(1003), " (period)" },
-    { QCborTag(22098), " (hint that indicates an additional level of indirection)" },
-    { QCborTag(55799), " (Self-describe CBOR; see Section 2.4.5 [RFC7049])" },
-    { QCborTag(15309736), " (RAINS Message)" },
+    { QCborTag(40), " (Multi-dimensional Array, row-major order [RFC8746])" },
+    { QCborTag(41), " (Homogeneous Array [RFC8746])" },
+    { QCborTag(42), " (IPLD content identifier)" },
+    { QCborTag(43), " (YANG bits datatype; see Section 6.7. [RFC9254])" },
+    { QCborTag(44), " (YANG enumeration datatype; see Section 6.6. [RFC9254])" },
+    { QCborTag(45), " (YANG identityref datatype; see Section 6.10. [RFC9254])" },
+    { QCborTag(46), " (YANG instance-identifier datatype; see Section 6.13. [RFC9254])" },
+    { QCborTag(47), " (YANG Schema Item iDentifier (sid); see Section 3.2. [RFC9254])" },
+    { QCborTag(52), " (IPv4, [prefixlen,IPv4], [IPv4,prefixpart] [RFC9164])" },
+    { QCborTag(54), " (IPv6, [prefixlen,IPv6], [IPv6,prefixpart] [RFC9164])" },
+    { QCborTag(61), " (CBOR Web Token (CWT) [RFC8392])" },
+    { QCborTag(63), " (Encoded CBOR Sequence [RFC8742])" },
+    { QCborTag(64), " (uint8 Typed Array [RFC8746])" },
+    { QCborTag(65), " (uint16, big endian, Typed Array [RFC8746])" },
+    { QCborTag(66), " (uint32, big endian, Typed Array [RFC8746])" },
+    { QCborTag(67), " (uint64, big endian, Typed Array [RFC8746])" },
+    { QCborTag(68), " (uint8 Typed Array, clamped arithmetic [RFC8746])" },
+    { QCborTag(69), " (uint16, little endian, Typed Array [RFC8746])" },
+    { QCborTag(70), " (uint32, little endian, Typed Array [RFC8746])" },
+    { QCborTag(71), " (uint64, little endian, Typed Array [RFC8746])" },
+    { QCborTag(72), " (sint8 Typed Array [RFC8746])" },
+    { QCborTag(73), " (sint16, big endian, Typed Array [RFC8746])" },
+    { QCborTag(74), " (sint32, big endian, Typed Array [RFC8746])" },
+    { QCborTag(75), " (sint64, big endian, Typed Array [RFC8746])" },
+    { QCborTag(76), " ((reserved) [RFC8746])" },
+    { QCborTag(77), " (sint16, little endian, Typed Array [RFC8746])" },
+    { QCborTag(78), " (sint32, little endian, Typed Array [RFC8746])" },
+    { QCborTag(79), " (sint64, little endian, Typed Array [RFC8746])" },
+    { QCborTag(80), " (IEEE 754 binary16, big endian, Typed Array [RFC8746])" },
+    { QCborTag(81), " (IEEE 754 binary32, big endian, Typed Array [RFC8746])" },
+    { QCborTag(82), " (IEEE 754 binary64, big endian, Typed Array [RFC8746])" },
+    { QCborTag(83), " (IEEE 754 binary128, big endian, Typed Array [RFC8746])" },
+    { QCborTag(84), " (IEEE 754 binary16, little endian, Typed Array [RFC8746])" },
+    { QCborTag(85), " (IEEE 754 binary32, little endian, Typed Array [RFC8746])" },
+    { QCborTag(86), " (IEEE 754 binary64, little endian, Typed Array [RFC8746])" },
+    { QCborTag(87), " (IEEE 754 binary128, little endian, Typed Array [RFC8746])" },
+    { QCborTag(96), " (COSE Encrypted Data Object [RFC9052])" },
+    { QCborTag(97), " (COSE MACed Data Object [RFC9052])" },
+    { QCborTag(98), " (COSE Signed Data Object [RFC9052])" },
+    { QCborTag(100), " (Number of days since the epoch date 1970-01-01 [RFC8943])" },
+    { QCborTag(101), " (alternatives as given by the uint + 128; see Section 9.1)" },
+    { QCborTag(103), " (Geographic Coordinates)" },
+    { QCborTag(104), " (Geographic Coordinate Reference System WKT or EPSG number)" },
+    { QCborTag(110), " (relative object identifier (BER encoding); SDNV sequence [RFC9090])" },
+    { QCborTag(111), " (object identifier (BER encoding) [RFC9090])" },
+    { QCborTag(112), " (object identifier (BER encoding), relative to 1.3.6.1.4.1 [RFC9090])" },
+    { QCborTag(120), " (Internet of Things Data Point)" },
+    { QCborTag(260),
+      " (Network Address (IPv4 or IPv6 or MAC Address) (DEPRECATED in favor of 52 and 54 for IP"
+      " addresses) [RFC9164])" },
+    { QCborTag(261),
+      " (Network Address Prefix (IPv4 or IPv6 Address + Mask Length) (DEPRECATED in favor of 52"
+      " and 54 for IP addresses) [RFC9164])" },
+    { QCborTag(271),
+      " (DDoS Open Threat Signaling (DOTS) signal channel object, as defined in [RFC9132])" },
+    { QCborTag(1004), " (full-date string [RFC8943])" },
+    { QCborTag(1040), " (Multi-dimensional Array, column-major order [RFC8746])" },
+    { QCborTag(55799), " (Self-described CBOR; see Section 3.4.6 [RFC8949])" },
+    { QCborTag(55800), " (indicates that the file contains CBOR Sequences [RFC9277])" },
+    { QCborTag(55801),
+      " (indicates that the file starts with a CBOR-Labeled Non-CBOR Data label. [RFC9277])" },
     { QCborTag(-1), nullptr }
 };
 // END GENERATED CODE
 
 enum {
     // See RFC 7049 section 2.
-    SmallValueBitLength     = 5,
-    SmallValueMask          = (1 << SmallValueBitLength) - 1,      /* 0x1f */
-    Value8Bit               = 24,
-    Value16Bit              = 25,
-    Value32Bit              = 26,
-    Value64Bit              = 27
+    SmallValueBitLength = 5,
+    SmallValueMask = (1 << SmallValueBitLength) - 1, /* 0x1f */
+    Value8Bit = 24,
+    Value16Bit = 25,
+    Value32Bit = 26,
+    Value64Bit = 27
 };
 
 //! [0]
 struct CborDumper
 {
-    enum DumpOption {
-        ShowCompact             = 0x01,
-        ShowWidthIndicators     = 0x02,
-        ShowAnnotated           = 0x04
-    };
+    enum DumpOption { ShowCompact = 0x01, ShowWidthIndicators = 0x02, ShowAnnotated = 0x04 };
     Q_DECLARE_FLAGS(DumpOptions, DumpOption)
 
     CborDumper(QFile *f, DumpOptions opts_);
@@ -131,8 +181,7 @@ static int cborNumberSize(quint64 value)
     return normalSize;
 }
 
-CborDumper::CborDumper(QFile *f, DumpOptions opts_)
-    : opts(opts_)
+CborDumper::CborDumper(QFile *f, DumpOptions opts_) : opts(opts_)
 {
     // try to mmap the file, this is faster
     char *ptr = reinterpret_cast<char *>(f->map(0, f->size(), QFile::MapPrivateOption));
@@ -179,51 +228,54 @@ QCborError CborDumper::dump()
     return err;
 }
 
-template <typename T> static inline bool canConvertTo(double v)
+template<typename T>
+static inline bool canConvertTo(double v)
 {
+    using TypeInfo = std::numeric_limits<T>;
     // The [conv.fpint] (7.10 Floating-integral conversions) section of the
     // standard says only exact conversions are guaranteed. Converting
     // integrals to floating-point with loss of precision has implementation-
     // defined behavior whether the next higher or next lower is returned;
     // converting FP to integral is UB if it can't be represented.;
-    static_assert(std::numeric_limits<T>::is_integer);
+    static_assert(TypeInfo::is_integer);
 
-    double supremum = ldexp(1, std::numeric_limits<T>::digits);
+    double supremum = ldexp(1, TypeInfo::digits);
     if (v >= supremum)
         return false;
 
-    if (v < std::numeric_limits<T>::min()) // either zero or a power of two, so it's exact
+    if (v < TypeInfo::min()) // either zero or a power of two, so it's exact
         return false;
 
     // we're in range
     return v == floor(v);
 }
 
-static QString fpToString(double v, const char *suffix)
+static QString fpToString(double v, QLatin1StringView suffix = ""_L1)
 {
     if (qIsInf(v))
-        return v < 0 ? QStringLiteral("-inf") : QStringLiteral("inf");
+        return v < 0 ? "-inf"_L1 : "inf"_L1;
     if (qIsNaN(v))
-        return QStringLiteral("nan");
+        return "nan"_L1;
     if (canConvertTo<qint64>(v))
-        return QString::number(qint64(v)) + ".0" + suffix;
+        return QString::number(qint64(v)) + ".0"_L1 + suffix;
     if (canConvertTo<quint64>(v))
-        return QString::number(quint64(v)) + ".0" + suffix;
+        return QString::number(quint64(v)) + ".0"_L1 + suffix;
 
     QString s = QString::number(v, 'g', QLocale::FloatingPointShortest);
-    if (!s.contains('.') && !s.contains('e'))
-        s += '.';
-    s += suffix;
+    if (!s.contains(u'.') && !s.contains(u'e'))
+        s += u'.';
+    if (suffix.size())
+        s += suffix;
     return s;
 };
 
 void CborDumper::dumpOne(int nestingLevel)
 {
-    QString indent(1, QLatin1Char(' '));
+    QString indent(1, u' ');
     QString indented = indent;
     if (!opts.testFlag(ShowCompact)) {
-        indent = QLatin1Char('\n') + QString(4 * nestingLevel, QLatin1Char(' '));
-        indented = QLatin1Char('\n') + QString(4 + 4 * nestingLevel, QLatin1Char(' '));
+        indent = u'\n' + QString(4 * nestingLevel, u' ');
+        indented = u'\n' + QString(4 + 4 * nestingLevel, u' ');
     }
 
     switch (reader.type()) {
@@ -263,7 +315,7 @@ void CborDumper::dumpOne(int nestingLevel)
                 printStringWidthIndicator(r.data.size());
 
                 r = reader.readByteArray();
-                comma = QLatin1Char(',') + indented;
+                comma = u',' + indented;
             }
         } else {
             auto r = reader.readString();
@@ -272,7 +324,7 @@ void CborDumper::dumpOne(int nestingLevel)
                 printStringWidthIndicator(r.data.toUtf8().size());
 
                 r = reader.readString();
-                comma = QLatin1Char(',') + indented;
+                comma = u',' + indented;
             }
         }
 
@@ -328,7 +380,7 @@ void CborDumper::dumpOne(int nestingLevel)
         if (reader.next()) {
             printWidthIndicator(quint64(tag));
             printf("(");
-            dumpOne(nestingLevel);  // same level!
+            dumpOne(nestingLevel); // same level!
             printf(")");
         }
 
@@ -360,15 +412,15 @@ void CborDumper::dumpOne(int nestingLevel)
         break;
 
     case QCborStreamReader::Float16:
-        printf("%s", qPrintable(fpToString(reader.toFloat16(), "f16")));
+        printf("%s", qPrintable(fpToString(reader.toFloat16(), "f16"_L1)));
         reader.next();
         break;
     case QCborStreamReader::Float:
-        printf("%s", qPrintable(fpToString(reader.toFloat(), "f")));
+        printf("%s", qPrintable(fpToString(reader.toFloat(), "f"_L1)));
         reader.next();
         break;
     case QCborStreamReader::Double:
-        printf("%s", qPrintable(fpToString(reader.toDouble(), "")));
+        printf("%s", qPrintable(fpToString(reader.toDouble())));
         reader.next();
         break;
     case QCborStreamReader::Invalid:
@@ -421,13 +473,14 @@ void CborDumper::dumpOneDetailed(int nestingLevel)
     };
 
     auto printFp = [=](const char *descr, double d) {
-        QString s = fpToString(d, "");
+        QString s = fpToString(d);
         if (s.size() <= 6)
             return print(descr, "%s", qPrintable(s));
         return print(descr, "%a", d);
     };
 
     auto printString = [=](const char *descr) {
+        constexpr qsizetype ChunkSizeLimit = std::numeric_limits<int>::max();
         QByteArray indent(nestingLevel * 2, ' ');
         const char *chunkStr = (reader.isLengthKnown() ? "" : "chunk ");
         int width = 48 - indent.size();
@@ -435,8 +488,8 @@ void CborDumper::dumpOneDetailed(int nestingLevel)
 
         qsizetype size = reader.currentStringChunkSize();
         if (size < 0)
-            return;         // error
-        if (size >= std::numeric_limits<int>::max()) {
+            return; // error
+        if (size >= ChunkSizeLimit) {
             fprintf(stderr, "String length too big, %lli\n", qint64(size));
             exit(EXIT_FAILURE);
         }
@@ -480,7 +533,7 @@ void CborDumper::dumpOneDetailed(int nestingLevel)
                 printf("  %s%s", indent.constData(), section.toHex(' ').constData());
 
                 // print the decode
-                QByteArray spaces(width > 0 ? width - section.size() * 3 + 1: 0, ' ');
+                QByteArray spaces(width > 0 ? width - section.size() * 3 + 1 : 0, ' ');
                 printf("%s # \"", spaces.constData());
                 auto ptr = reinterpret_cast<const uchar *>(section.constData());
                 for (int j = 0; j < section.size(); ++j)
@@ -492,8 +545,8 @@ void CborDumper::dumpOneDetailed(int nestingLevel)
             // get the next chunk
             size = reader.currentStringChunkSize();
             if (size < 0)
-                return;         // error
-            if (size >= std::numeric_limits<int>::max()) {
+                return; // error
+            if (size >= ChunkSizeLimit) {
                 fprintf(stderr, "String length too big, %lli\n", qint64(size));
                 exit(EXIT_FAILURE);
             }
@@ -631,7 +684,9 @@ void CborDumper::printByteArray(const QByteArray &ba)
         break;
 
     case quint8(QCborKnownTags::ExpectedBase64url):
-        printf("b64'%s'", ba.toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals).constData());
+        printf("b64'%s'",
+               ba.toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals)
+                       .constData());
         break;
     }
 }
@@ -672,23 +727,20 @@ int main(int argc, char *argv[])
     setlocale(LC_ALL, "C");
 
     QCommandLineParser parser;
-    parser.setApplicationDescription(QStringLiteral("CBOR Dumper tool"));
+    parser.setApplicationDescription("CBOR Dumper tool"_L1);
     parser.addHelpOption();
 
-    QCommandLineOption compact({QStringLiteral("c"), QStringLiteral("compact")},
-                               QStringLiteral("Use compact form (no line breaks)"));
+    QCommandLineOption compact({"c"_L1, "compact"_L1}, "Use compact form (no line breaks)"_L1);
     parser.addOption(compact);
 
-    QCommandLineOption showIndicators({QStringLiteral("i"), QStringLiteral("indicators")},
-                                      QStringLiteral("Show indicators for width of lengths and integrals"));
+    QCommandLineOption showIndicators({ "i"_L1, "indicators"_L1 },
+                                      "Show indicators for width of lengths and integrals"_L1);
     parser.addOption(showIndicators);
 
-    QCommandLineOption verbose({QStringLiteral("a"), QStringLiteral("annotated")},
-                               QStringLiteral("Show bytes and annotated decoding"));
+    QCommandLineOption verbose({"a"_L1, "annotated"_L1}, "Show bytes and annotated decoding"_L1);
     parser.addOption(verbose);
 
-    parser.addPositionalArgument(QStringLiteral("[source]"),
-                                 QStringLiteral("CBOR file to read from"));
+    parser.addPositionalArgument("[source]"_L1, "CBOR file to read from"_L1);
 
     parser.process(app);
 
