@@ -6,20 +6,17 @@
 
 #include <QWidget>
 
-QT_BEGIN_NAMESPACE
-class QSlider;
-class QPushButton;
-QT_END_NAMESPACE
+QT_FORWARD_DECLARE_CLASS(QSlider)
+QT_FORWARD_DECLARE_CLASS(QPushButton)
 
 class GLWidget;
-class MainWindow;
 
 class Window : public QWidget
 {
     Q_OBJECT
 
 public:
-    Window(MainWindow *mw);
+    Window();
 
 protected:
     void keyPressEvent(QKeyEvent *event) override;
@@ -28,6 +25,8 @@ private slots:
     void dockUndock();
 
 private:
+    void dock();
+    void undock();
     QSlider *createSlider();
 
     GLWidget *glWidget;
@@ -35,7 +34,6 @@ private:
     QSlider *ySlider;
     QSlider *zSlider;
     QPushButton *dockBtn;
-    MainWindow *mainWindow;
 };
 
 #endif
