@@ -14,6 +14,7 @@ import android.content.pm.PackageManager;
 import android.content.pm.PackageManager.NameNotFoundException;
 import android.content.res.AssetManager;
 import android.content.res.Configuration;
+import android.graphics.Color;
 import android.graphics.drawable.ColorDrawable;
 import android.graphics.drawable.Drawable;
 import android.graphics.Rect;
@@ -43,6 +44,8 @@ import android.view.Surface;
 import android.view.View;
 import android.view.ViewConfiguration;
 import android.view.ViewGroup;
+import android.view.Window;
+import android.view.WindowInsetsController;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
 import android.view.ViewTreeObserver;
@@ -85,6 +88,7 @@ public class QtActivityDelegate
     private static final String APPLICATION_PARAMETERS_KEY = "application.parameters";
     private static final String STATIC_INIT_CLASSES_KEY = "static.init.classes";
 
+    // Keep in sync with QtAndroid::SystemUiVisibility in androidjnimain.h
     public static final int SYSTEM_UI_VISIBILITY_NORMAL = 0;
     public static final int SYSTEM_UI_VISIBILITY_FULLSCREEN = 1;
     public static final int SYSTEM_UI_VISIBILITY_TRANSLUCENT = 2;
@@ -131,12 +135,12 @@ public class QtActivityDelegate
 
         m_systemUiVisibility = systemUiVisibility;
 
-        int systemUiVisibilityFlags = 0;
+        int systemUiVisibilityFlags = View.SYSTEM_UI_FLAG_VISIBLE;
         switch (m_systemUiVisibility) {
         case SYSTEM_UI_VISIBILITY_NORMAL:
             m_activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
             m_activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
-            systemUiVisibilityFlags = View.SYSTEM_UI_FLAG_VISIBLE;
+            setDisplayCutoutLayout(WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_NEVER);
             break;
         case SYSTEM_UI_VISIBILITY_FULLSCREEN:
             m_activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
@@ -148,13 +152,14 @@ public class QtActivityDelegate
                     | View.SYSTEM_UI_FLAG_FULLSCREEN
                     | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                     | View.INVISIBLE;
+            setDisplayCutoutLayout(WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_DEFAULT);
             break;
         case SYSTEM_UI_VISIBILITY_TRANSLUCENT:
             m_activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN
                     | WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION
                     | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
             m_activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
-            systemUiVisibilityFlags = View.SYSTEM_UI_FLAG_VISIBLE;
+            setDisplayCutoutLayout(WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_ALWAYS);
             break;
         };
 
@@ -163,6 +168,12 @@ public class QtActivityDelegate
         m_layout.requestLayout();
     }
 
+    private void setDisplayCutoutLayout(int cutoutLayout)
+    {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P)
+            m_activity.getWindow().getAttributes().layoutInDisplayCutoutMode = cutoutLayout;
+    }
+
     public void updateFullScreen()
     {
         if (m_systemUiVisibility == SYSTEM_UI_VISIBILITY_FULLSCREEN) {
@@ -559,7 +570,9 @@ public class QtActivityDelegate
     private final DisplayManager.DisplayListener displayListener = new DisplayManager.DisplayListener()
     {
         @Override
-        public void onDisplayAdded(int displayId) { }
+        public void onDisplayAdded(int displayId) {
+            QtNative.handleScreenAdded(displayId);
+        }
 
         private boolean isSimilarRotation(int r1, int r2)
         {
@@ -586,10 +599,13 @@ public class QtActivityDelegate
 
             float refreshRate = display.getRefreshRate();
             QtNative.handleRefreshRateChanged(refreshRate);
+            QtNative.handleScreenChanged(displayId);
         }
 
         @Override
-        public void onDisplayRemoved(int displayId) { }
+        public void onDisplayRemoved(int displayId) {
+            QtNative.handleScreenRemoved(displayId);
+        }
     };
 
     public boolean updateActivity(Activity activity)
@@ -713,7 +729,7 @@ public class QtActivityDelegate
                                         "Droid Sans Mono;Droid Sans;Droid Sans Fallback");
         QtNative.setEnvironmentVariable("QT_ANDROID_FONTS_SERIF", "Droid Serif");
         QtNative.setEnvironmentVariable("HOME", m_activity.getFilesDir().getAbsolutePath());
-        QtNative.setEnvironmentVariable("TMPDIR", m_activity.getFilesDir().getAbsolutePath());
+        QtNative.setEnvironmentVariable("TMPDIR", m_activity.getCacheDir().getAbsolutePath());
         QtNative.setEnvironmentVariable("QT_ANDROID_FONTS",
                                         "Roboto;Droid Sans;Droid Sans Fallback");
         QtNative.setEnvironmentVariable("QT_ANDROID_APP_ICON_SIZE",
@@ -973,6 +989,19 @@ public class QtActivityDelegate
 
     private void handleUiModeChange(int uiMode)
     {
+        // QTBUG-108365
+        if (Build.VERSION.SDK_INT >= 30) {
+            // Since 29 version we are using Theme_DeviceDefault_DayNight
+            Window window = m_activity.getWindow();
+            WindowInsetsController controller = window.getInsetsController();
+            if (controller != null) {
+                // set APPEARANCE_LIGHT_STATUS_BARS if needed
+                int appearanceLight = Color.luminance(window.getStatusBarColor()) > 0.5 ?
+                        WindowInsetsController.APPEARANCE_LIGHT_STATUS_BARS : 0;
+                controller.setSystemBarsAppearance(appearanceLight,
+                    WindowInsetsController.APPEARANCE_LIGHT_STATUS_BARS);
+            }
+        }
         switch (uiMode) {
             case Configuration.UI_MODE_NIGHT_NO:
                 ExtractStyle.runIfNeeded(m_activity, false);
