@@ -2,297 +2,46 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qdeadlinetimer.h"
-#include "qdeadlinetimer_p.h"
 #include "private/qnumeric_p.h"
 
 QT_BEGIN_NAMESPACE
 
 QT_IMPL_METATYPE_EXTERN(QDeadlineTimer)
 
+using namespace std::chrono;
+
 namespace {
-    class TimeReference
-    {
-        enum : unsigned {
-            umega = 1000 * 1000,
-            ugiga = umega * 1000
-        };
-
-        enum : qint64 {
-            kilo = 1000,
-            mega = kilo * 1000,
-            giga = mega * 1000
-        };
-
-    public:
-        enum RoundingStrategy {
-            RoundDown,
-            RoundUp,
-            RoundDefault = RoundDown
-        };
-
-        static constexpr qint64 Min = std::numeric_limits<qint64>::min();
-        static constexpr qint64 Max = std::numeric_limits<qint64>::max();
-
-        inline TimeReference(qint64 = 0, unsigned = 0);
-        inline void updateTimer(qint64 &, unsigned &);
-
-        inline bool addNanoseconds(qint64);
-        inline bool addMilliseconds(qint64);
-        bool addSecsAndNSecs(qint64, qint64);
-
-        inline bool subtract(const qint64, const unsigned);
-
-        inline bool toMilliseconds(qint64 *, RoundingStrategy = RoundDefault) const;
-        inline bool toNanoseconds(qint64 *) const;
-
-        inline void saturate(bool toMax);
-        static bool sign(qint64, qint64);
-
-    private:
-        bool adjust(const qint64, const unsigned, qint64 = 0);
-
-    private:
-        qint64 secs;
-        unsigned nsecs;
-    };
+struct TimeReference : std::numeric_limits<qint64>
+{
+    static constexpr qint64 Min = min();
+    static constexpr qint64 Max = max();
+};
 }
 
-inline TimeReference::TimeReference(qint64 t1, unsigned t2)
-    : secs(t1), nsecs(t2)
+template <typename Duration1, typename... Durations>
+static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
 {
-}
+    qint64 v = dur.count();
+    qint64 saturated = std::numeric_limits<qint64>::max();
+    if (v < 0)
+        saturated = std::numeric_limits<qint64>::min();
 
-inline void TimeReference::updateTimer(qint64 &t1, unsigned &t2)
-{
-    t1 = secs;
-    t2 = nsecs;
-}
+    // convert to nanoseconds with saturation
+    using Ratio = std::ratio_divide<typename Duration1::period, nanoseconds::period>;
+    static_assert(Ratio::den == 1, "sub-multiples of nanosecond are not supported");
+    if (qMulOverflow<Ratio::num>(v, &v))
+        return saturated;
 
-inline void TimeReference::saturate(bool toMax)
-{
-    secs = toMax ? Max : Min;
-}
-
-/*!
- * \internal
- *
- * Determines the sign of a (seconds, nanoseconds) pair
- * for differentiating overflow from underflow. It doesn't
- * deal with equality as it shouldn't ever be called in that case.
- *
- * Returns true if the pair represents a positive time offset
- * false otherwise.
- */
-bool TimeReference::sign(qint64 secs, qint64 nsecs)
-{
-    if (secs > 0) {
-        if (nsecs > 0)
-            return true;
-    } else {
-        if (nsecs < 0)
-            return false;
+    qint64 r;
+    if (qAddOverflow(t1, v, &r))
+        return saturated;
+    if constexpr (sizeof...(Durations)) {
+        // chain more additions
+        return add_saturate(r, extra...);
     }
-
-    // They are different in sign
-    secs += nsecs / giga;
-    if (secs > 0)
-        return true;
-    else if (secs < 0)
-        return false;
-
-    // We should never get over|underflow out of
-    // the case: secs * giga == -nsecs
-    // So the sign of nsecs is the deciding factor
-    Q_ASSERT(nsecs % giga != 0);
-    return nsecs > 0;
+    return r;
 }
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_DARWIN)
-inline bool TimeReference::addNanoseconds(qint64 arg)
-{
-    return addSecsAndNSecs(arg / giga, arg % giga);
-}
-
-inline bool TimeReference::addMilliseconds(qint64 arg)
-{
-    return addSecsAndNSecs(arg / kilo, (arg % kilo) * mega);
-}
-
-/*!
- * \internal
- *
- * Adds \a t1 addSecs seconds and \a addNSecs nanoseconds to the
- * time reference. The arguments are normalized to seconds (qint64)
- * and nanoseconds (unsigned) before the actual calculation is
- * delegated to adjust(). If the nanoseconds are negative the
- * owed second used for the normalization is passed on to adjust()
- * as third argument.
- *
- * Returns true if operation was successful, false on over|underflow
- */
-bool TimeReference::addSecsAndNSecs(qint64 addSecs, qint64 addNSecs)
-{
-    // Normalize the arguments
-    if (qAbs(addNSecs) >= giga) {
-        if (add_overflow<qint64>(addSecs, addNSecs / giga, &addSecs))
-            return false;
-
-        addNSecs %= giga;
-    }
-
-    if (addNSecs < 0)
-        return adjust(addSecs, ugiga - unsigned(-addNSecs), -1);
-
-    return adjust(addSecs, unsigned(addNSecs));
-}
-
-/*!
- * \internal
- *
- * Adds \a t1 seconds and \a t2 nanoseconds to the internal members.
- * Takes into account the additional \a carrySeconds we may owe or need to carry over.
- *
- * Returns true if operation was successful, false on over|underflow
- */
-bool TimeReference::adjust(const qint64 t1, const unsigned t2, qint64 carrySeconds)
-{
-    static_assert(QDeadlineTimerNanosecondsInT2);
-    nsecs += t2;
-    if (nsecs >= ugiga) {
-        nsecs -= ugiga;
-        carrySeconds++;
-    }
-
-    // We don't worry about the order of addition, because the result returned by
-    // callers of this function is unchanged regardless of us over|underflowing.
-    // If we do, we do so by no more than a second, thus saturating the timer to
-    // Forever has the same effect as if we did the arithmetic exactly and salvaged
-    // the overflow.
-    return !add_overflow<qint64>(secs, t1, &secs) && !add_overflow<qint64>(secs, carrySeconds, &secs);
-}
-
-/*!
- * \internal
- *
- * Subtracts \a t1 seconds and \a t2 nanoseconds from the time reference.
- * When normalizing the nanoseconds to a positive number the owed seconds is
- * passed as third argument to adjust() as the seconds may over|underflow
- * if we do the calculation directly. There is little sense to check the
- * seconds for over|underflow here in case we are going to need to carry
- * over a second _after_ we add the nanoseconds.
- *
- * Returns true if operation was successful, false on over|underflow
- */
-inline bool TimeReference::subtract(const qint64 t1, const unsigned t2)
-{
-    Q_ASSERT(t2 < ugiga);
-    return adjust(-t1, ugiga - t2, -1);
-}
-
-/*!
- * \internal
- *
- * Converts the time reference to milliseconds.
- *
- * Checks are done without making use of mul_overflow because it may
- * not be implemented on some 32bit platforms.
- *
- * Returns true if operation was successful, false on over|underflow
- */
-inline bool TimeReference::toMilliseconds(qint64 *result, RoundingStrategy rounding) const
-{
-    static constexpr qint64 maxSeconds = Max / kilo;
-    static constexpr qint64 minSeconds = Min / kilo;
-    if (secs > maxSeconds || secs < minSeconds)
-        return false;
-
-    unsigned ns = rounding == RoundDown ? nsecs : nsecs + umega - 1;
-
-    return !add_overflow<qint64>(secs * kilo, ns / umega, result);
-}
-
-/*!
- * \internal
- *
- * Converts the time reference to nanoseconds.
- *
- * Checks are done without making use of mul_overflow because it may
- * not be implemented on some 32bit platforms.
- *
- * Returns true if operation was successful, false on over|underflow
- */
-inline bool TimeReference::toNanoseconds(qint64 *result) const
-{
-    static constexpr qint64 maxSeconds = Max / giga;
-    static constexpr qint64 minSeconds = Min / giga;
-    if (secs > maxSeconds || secs < minSeconds)
-        return false;
-
-    return !add_overflow<qint64>(secs * giga, nsecs, result);
-}
-#else
-inline bool TimeReference::addNanoseconds(qint64 arg)
-{
-    return adjust(arg, 0);
-}
-
-inline bool TimeReference::addMilliseconds(qint64 arg)
-{
-    static constexpr qint64 maxMilliseconds = Max / mega;
-    if (qAbs(arg) > maxMilliseconds)
-        return false;
-
-    return addNanoseconds(arg * mega);
-}
-
-inline bool TimeReference::addSecsAndNSecs(qint64 addSecs, qint64 addNSecs)
-{
-    static constexpr qint64 maxSeconds = Max / giga;
-    static constexpr qint64 minSeconds = Min / giga;
-    if (addSecs > maxSeconds || addSecs < minSeconds || add_overflow<qint64>(addSecs * giga, addNSecs, &addNSecs))
-        return false;
-
-    return addNanoseconds(addNSecs);
-}
-
-inline bool TimeReference::adjust(const qint64 t1, const unsigned t2, qint64 carrySeconds)
-{
-    static_assert(!QDeadlineTimerNanosecondsInT2);
-    Q_UNUSED(t2);
-    Q_UNUSED(carrySeconds);
-
-    return !add_overflow<qint64>(secs, t1, &secs);
-}
-
-inline bool TimeReference::subtract(const qint64 t1, const unsigned t2)
-{
-    Q_UNUSED(t2);
-
-    return addNanoseconds(-t1);
-}
-
-inline bool TimeReference::toMilliseconds(qint64 *result, RoundingStrategy rounding) const
-{
-    // Force QDeadlineTimer to treat the border cases as
-    // over|underflow and saturate the results returned to the user.
-    // We don't want to get valid milliseconds out of saturated timers.
-    if (secs == Max || secs == Min)
-        return false;
-
-    *result = secs / mega;
-    if (rounding == RoundUp && secs > *result * mega)
-        (*result)++;
-
-    return true;
-}
-
-inline bool TimeReference::toNanoseconds(qint64 *result) const
-{
-    *result = secs;
-    return true;
-}
-#endif
-
 /*!
     \class QDeadlineTimer
     \inmodule QtCore
@@ -382,10 +131,12 @@ inline bool TimeReference::toNanoseconds(qint64 *result) const
 */
 
 /*!
+    \fn QDeadlineTimer::QDeadlineTimer()
     \fn QDeadlineTimer::QDeadlineTimer(Qt::TimerType timerType)
 
     Constructs an expired QDeadlineTimer object. For this object,
-    remainingTime() will return 0.
+    remainingTime() will return 0. If \a timerType is not set, then the object
+    will use the \l{Qt::CoarseTimer}{coarse} \l{QDeadlineTimer#Timer types}{timer type}.
 
     The timer type \a timerType may be ignored, since the timer is already
     expired. Similarly, for optimization purposes, this function will not
@@ -415,7 +166,7 @@ inline bool TimeReference::toNanoseconds(qint64 *result) const
     from the moment of the creation of this object, if msecs is positive. If \a
     msecs is zero, this QDeadlineTimer will be marked as expired, causing
     remainingTime() to return zero and deadline() to return an indeterminate
-    time point in the past. If \a msecs is -1, the timer will be set to never
+    time point in the past. If \a msecs is negative, the timer will be set to never
     expire, causing remainingTime() to return -1 and deadline() to return the
     maximum value.
 
@@ -428,10 +179,12 @@ inline bool TimeReference::toNanoseconds(qint64 *result) const
     functionality is required, use QDeadlineTimer::current() and add time to
     it.
 
+    \note Prior to Qt 6.6, the only value that caused the timer to never expire
+    was -1.
+
     \sa hasExpired(), isForever(), remainingTime(), setRemainingTime()
 */
 QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) noexcept
-    : t2(0)
 {
     setRemainingTime(msecs, type);
 }
@@ -495,51 +248,70 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) noexcept
 /*!
     Sets the remaining time for this QDeadlineTimer object to \a msecs
     milliseconds from now, if \a msecs has a positive value. If \a msecs is
-    zero, this QDeadlineTimer object will be marked as expired, whereas a value
-    of -1 will set it to never expire.
+    zero, this QDeadlineTimer object will be marked as expired, whereas a
+    negative value will set it to never expire.
+
+    For optimization purposes, if \a msecs is zero, this function may skip
+    obtaining the current time and may instead use a value known to be in the
+    past. If that happens, deadline() may return an unexpected value and this
+    object cannot be used in calculation of how long it is overdue. If that
+    functionality is required, use QDeadlineTimer::current() and add time to
+    it.
 
     The timer type for this QDeadlineTimer object will be set to the specified \a timerType.
 
+    \note Prior to Qt 6.6, the only value that caused the timer to never expire
+    was -1.
+
     \sa setPreciseRemainingTime(), hasExpired(), isForever(), remainingTime()
 */
 void QDeadlineTimer::setRemainingTime(qint64 msecs, Qt::TimerType timerType) noexcept
 {
-    if (msecs == -1) {
+    if (msecs < 0) {
         *this = QDeadlineTimer(Forever, timerType);
-        return;
+    } else if (msecs == 0) {
+        *this = QDeadlineTimer(timerType);
+        t1 = std::numeric_limits<qint64>::min();
+    } else {
+        *this = current(timerType);
+        milliseconds ms(msecs);
+        t1 = add_saturate(t1, ms);
     }
-
-    *this = current(timerType);
-
-    TimeReference ref(t1, t2);
-    if (!ref.addMilliseconds(msecs))
-        ref.saturate(msecs > 0);
-    ref.updateTimer(t1, t2);
 }
 
 /*!
     Sets the remaining time for this QDeadlineTimer object to \a secs seconds
     plus \a nsecs nanoseconds from now, if \a secs has a positive value. If \a
-    secs is -1, this QDeadlineTimer will be set it to never expire. If both
-    parameters are zero, this QDeadlineTimer will be marked as expired.
+    secs is negative, this QDeadlineTimer will be set it to never expire (this
+    behavior does not apply to \a nsecs). If both parameters are zero, this
+    QDeadlineTimer will be marked as expired.
+
+    For optimization purposes, if both \a secs and \a nsecs are zero, this
+    function may skip obtaining the current time and may instead use a value
+    known to be in the past. If that happens, deadline() may return an
+    unexpected value and this object cannot be used in calculation of how long
+    it is overdue. If that functionality is required, use
+    QDeadlineTimer::current() and add time to it.
 
     The timer type for this QDeadlineTimer object will be set to the specified
     \a timerType.
 
+    \note Prior to Qt 6.6, the only condition that caused the timer to never
+    expire was when \a secs was -1.
+
     \sa setRemainingTime(), hasExpired(), isForever(), remainingTime()
 */
 void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::TimerType timerType) noexcept
 {
-    if (secs == -1) {
+    if (secs < 0) {
         *this = QDeadlineTimer(Forever, timerType);
-        return;
+    } else if (secs == 0 && nsecs == 0) {
+        *this = QDeadlineTimer(timerType);
+        t1 = std::numeric_limits<qint64>::min();
+    } else {
+        *this = current(timerType);
+        t1 = add_saturate(t1, seconds{secs}, nanoseconds{nsecs});
     }
-
-    *this = current(timerType);
-    TimeReference ref(t1, t2);
-    if (!ref.addSecsAndNSecs(secs, nsecs))
-        ref.saturate(TimeReference::sign(secs, nsecs));
-    ref.updateTimer(t1, t2);
 }
 
 /*!
@@ -589,6 +361,8 @@ bool QDeadlineTimer::hasExpired() const noexcept
 {
     if (isForever())
         return false;
+    if (t1 == std::numeric_limits<qint64>::min())
+        return true;
     return *this <= current(timerType());
 }
 
@@ -637,19 +411,8 @@ qint64 QDeadlineTimer::remainingTime() const noexcept
     if (isForever())
         return -1;
 
-    QDeadlineTimer now = current(timerType());
-    TimeReference ref(t1, t2);
-
-    qint64 msecs;
-    if (!ref.subtract(now.t1, now.t2))
-        return 0;   // We can only underflow here
-
-    // If we fail the conversion, t1 < now.t1 means we underflowed,
-    // thus the deadline had long expired
-    if (!ref.toMilliseconds(&msecs, TimeReference::RoundUp))
-        return t1 < now.t1 ? 0 : -1;
-
-    return msecs < 0 ? 0 : msecs;
+    nanoseconds nsecs(remainingTimeNSecs());
+    return ceil<milliseconds>(nsecs).count();
 }
 
 /*!
@@ -671,23 +434,19 @@ qint64 QDeadlineTimer::remainingTimeNSecs() const noexcept
 /*!
     \internal
     Same as remainingTimeNSecs, but may return negative remaining times. Does
-    not deal with Forever. In case of underflow the result is saturated to
-    the minimum possible value, on overflow  - the maximum possible value.
+    not deal with Forever. In case of underflow, which is only possible if the
+    timer has expired, an arbitrary negative value is returned.
 */
 qint64 QDeadlineTimer::rawRemainingTimeNSecs() const noexcept
 {
+    if (t1 == std::numeric_limits<qint64>::min())
+        return t1;          // we'd saturate to this anyway
+
     QDeadlineTimer now = current(timerType());
-    TimeReference ref(t1, t2);
-
-    qint64 nsecs;
-    if (!ref.subtract(now.t1, now.t2))
-        return TimeReference::Min;  // We can only underflow here
-
-    // If we fail the conversion, t1 < now.t1 means we underflowed,
-    // thus the deadline had long expired
-    if (!ref.toNanoseconds(&nsecs))
-        return t1 < now.t1 ? TimeReference::Min : TimeReference::Max;
-    return nsecs;
+    qint64 r;
+    if (qSubOverflow(t1, now.t1, &r))
+        return -1;      // any negative number is fine
+    return r;
 }
 
 /*!
@@ -714,12 +473,11 @@ qint64 QDeadlineTimer::deadline() const noexcept
 {
     if (isForever())
         return TimeReference::Max;
+    if (t1 == TimeReference::Min)
+        return t1;
 
-    qint64 result;
-    if (!TimeReference(t1, t2).toMilliseconds(&result))
-        return t1 < 0 ? TimeReference::Min : TimeReference::Max;
-
-    return result;
+    nanoseconds ns(t1);
+    return duration_cast<milliseconds>(ns).count();
 }
 
 /*!
@@ -748,11 +506,7 @@ qint64 QDeadlineTimer::deadlineNSecs() const noexcept
     if (isForever())
         return TimeReference::Max;
 
-    qint64 result;
-    if (!TimeReference(t1, t2).toNanoseconds(&result))
-        return t1 < 0 ? TimeReference::Min : TimeReference::Max;
-
-    return result;
+    return t1;
 }
 
 /*!
@@ -776,11 +530,7 @@ void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType) noexcept
     }
 
     type = timerType;
-
-    TimeReference ref;
-    if (!ref.addMilliseconds(msecs))
-        ref.saturate(msecs > 0);
-    ref.updateTimer(t1, t2);
+    t1 = add_saturate(0, milliseconds{msecs});
 }
 
 /*!
@@ -798,13 +548,7 @@ void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType) noexcept
 void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs, Qt::TimerType timerType) noexcept
 {
     type = timerType;
-
-    // We don't pass the seconds to the constructor, because we don't know
-    // at this point if t1 holds the seconds or nanoseconds; it's platform specific.
-    TimeReference ref;
-    if (!ref.addSecsAndNSecs(secs, nsecs))
-        ref.saturate(TimeReference::sign(secs, nsecs));
-    ref.updateTimer(t1, t2);
+    t1 = add_saturate(0, seconds{secs}, nanoseconds{nsecs});
 }
 
 /*!
@@ -820,11 +564,7 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) noexcep
     if (dt.isForever())
         return dt;
 
-    TimeReference ref(dt.t1, dt.t2);
-    if (!ref.addNanoseconds(nsecs))
-        ref.saturate(nsecs > 0);
-    ref.updateTimer(dt.t1, dt.t2);
-
+    dt.t1 = add_saturate(dt.t1, nanoseconds{nsecs});
     return dt;
 }
 
@@ -837,6 +577,17 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) noexcep
 
     The QDeadlineTimer object will be constructed with the specified \a timerType.
 */
+QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
+{
+    // ensure we get nanoseconds; this will work so long as steady_clock's
+    // time_point isn't of finer resolution (picoseconds)
+    std::chrono::nanoseconds ns = std::chrono::steady_clock::now().time_since_epoch();
+
+    QDeadlineTimer result;
+    result.t1 = ns.count();
+    result.type = timerType;
+    return result;
+}
 
 /*!
     \fn bool QDeadlineTimer::operator==(QDeadlineTimer d1, QDeadlineTimer d2)
@@ -931,11 +682,7 @@ QDeadlineTimer operator+(QDeadlineTimer dt, qint64 msecs)
     if (dt.isForever())
         return dt;
 
-    TimeReference ref(dt.t1, dt.t2);
-    if (!ref.addMilliseconds(msecs))
-        ref.saturate(msecs > 0);
-    ref.updateTimer(dt.t1, dt.t2);
-
+    dt.t1 = add_saturate(dt.t1, milliseconds{msecs});
     return dt;
 }
 
@@ -1001,11 +748,4 @@ QDeadlineTimer operator+(QDeadlineTimer dt, qint64 msecs)
   Returns the time remaining before the deadline.
  */
 
-/*!
-  \fn QPair<qint64, unsigned> QDeadlineTimer::_q_data() const
-  \internal
-*/
-
-// the rest of the functions are in qelapsedtimer_xxx.cpp
-
 QT_END_NAMESPACE
