@@ -17,6 +17,8 @@
 
 #include <sys/times.h>
 
+using namespace std::chrono;
+
 QT_BEGIN_NAMESPACE
 
 Q_CORE_EXPORT bool qt_disable_lowpriority_timers=false;
@@ -28,25 +30,6 @@ Q_CORE_EXPORT bool qt_disable_lowpriority_timers=false;
 
 QTimerInfoList::QTimerInfoList()
 {
-#if (_POSIX_MONOTONIC_CLOCK-0 <= 0) && !defined(Q_OS_MAC)
-    if (!QElapsedTimer::isMonotonic()) {
-        // not using monotonic timers, initialize the timeChanged() machinery
-        previousTime = qt_gettime();
-
-        tms unused;
-        previousTicks = times(&unused);
-
-        ticksPerSecond = sysconf(_SC_CLK_TCK);
-        msPerTick = 1000/ticksPerSecond;
-    } else {
-        // detected monotonic timers
-        previousTime.tv_sec = previousTime.tv_nsec = 0;
-        previousTicks = 0;
-        ticksPerSecond = 0;
-        msPerTick = 0;
-    }
-#endif
-
     firstTimerInfo = nullptr;
 }
 
@@ -55,85 +38,6 @@ timespec QTimerInfoList::updateCurrentTime()
     return (currentTime = qt_gettime());
 }
 
-#if ((_POSIX_MONOTONIC_CLOCK-0 <= 0) && !defined(Q_OS_MAC) && !defined(Q_OS_INTEGRITY)) || defined(QT_BOOTSTRAPPED)
-
-timespec qAbsTimespec(const timespec &t)
-{
-    timespec tmp = t;
-    if (tmp.tv_sec < 0) {
-        tmp.tv_sec = -tmp.tv_sec - 1;
-        tmp.tv_nsec -= 1000000000;
-    }
-    if (tmp.tv_sec == 0 && tmp.tv_nsec < 0) {
-        tmp.tv_nsec = -tmp.tv_nsec;
-    }
-    return normalizedTimespec(tmp);
-}
-
-/*
-  Returns \c true if the real time clock has changed by more than 10%
-  relative to the processor time since the last time this function was
-  called. This presumably means that the system time has been changed.
-
-  If /a delta is nonzero, delta is set to our best guess at how much the system clock was changed.
-*/
-bool QTimerInfoList::timeChanged(timespec *delta)
-{
-    struct tms unused;
-    clock_t currentTicks = times(&unused);
-
-    clock_t elapsedTicks = currentTicks - previousTicks;
-    timespec elapsedTime = currentTime - previousTime;
-
-    timespec elapsedTimeTicks;
-    elapsedTimeTicks.tv_sec = elapsedTicks / ticksPerSecond;
-    elapsedTimeTicks.tv_nsec = (((elapsedTicks * 1000) / ticksPerSecond) % 1000) * 1000 * 1000;
-
-    timespec dummy;
-    if (!delta)
-        delta = &dummy;
-    *delta = elapsedTime - elapsedTimeTicks;
-
-    previousTicks = currentTicks;
-    previousTime = currentTime;
-
-    // If tick drift is more than 10% off compared to realtime, we assume that the clock has
-    // been set. Of course, we have to allow for the tick granularity as well.
-    timespec tickGranularity;
-    tickGranularity.tv_sec = 0;
-    tickGranularity.tv_nsec = msPerTick * 1000 * 1000;
-    return elapsedTimeTicks < ((qAbsTimespec(*delta) - tickGranularity) * 10);
-}
-
-/*
-  repair broken timer
-*/
-void QTimerInfoList::timerRepair(const timespec &diff)
-{
-    // repair all timers
-    for (int i = 0; i < size(); ++i) {
-        QTimerInfo *t = at(i);
-        t->timeout = t->timeout + diff;
-    }
-}
-
-void QTimerInfoList::repairTimersIfNeeded()
-{
-    if (QElapsedTimer::isMonotonic())
-        return;
-    timespec delta;
-    if (timeChanged(&delta))
-        timerRepair(delta);
-}
-
-#else // !(_POSIX_MONOTONIC_CLOCK-0 <= 0) && !defined(QT_BOOTSTRAPPED)
-
-void QTimerInfoList::repairTimersIfNeeded()
-{
-}
-
-#endif
-
 /*
   insert timer info into list
 */
@@ -148,19 +52,6 @@ void QTimerInfoList::timerInsert(QTimerInfo *ti)
     insert(index+1, ti);
 }
 
-inline timespec &operator+=(timespec &t1, int ms)
-{
-    t1.tv_sec += ms / 1000;
-    t1.tv_nsec += ms % 1000 * 1000 * 1000;
-    return normalizedTimespec(t1);
-}
-
-inline timespec operator+(const timespec &t1, int ms)
-{
-    timespec t2 = t1;
-    return t2 += ms;
-}
-
 static constexpr timespec roundToMillisecond(timespec val)
 {
     // always round up
@@ -178,6 +69,25 @@ static_assert(roundToMillisecond({0, 1'000'000}) == timespec{0, 1'000'000});
 static_assert(roundToMillisecond({0, 999'999'999}) == timespec{1, 0});
 static_assert(roundToMillisecond({1, 0}) == timespec{1, 0});
 
+static constexpr seconds roundToSecs(milliseconds msecs)
+{
+    // The very coarse timer is based on full second precision, so we want to
+    // round the interval to the closest second, rounding 500ms up to 1s.
+    //
+    // std::chrono::round() wouldn't work with all multiples of 500 because for the
+    // middle point it would round to even:
+    // value  round()  wanted
+    // 500      0        1
+    // 1500     2        2
+    // 2500     2        3
+
+    auto secs = duration_cast<seconds>(msecs);
+    const milliseconds frac = msecs - secs;
+    if (frac >= 500ms)
+        ++secs;
+    return secs;
+}
+
 #ifdef QTIMERINFO_DEBUG
 QDebug operator<<(QDebug s, timeval tv)
 {
@@ -194,7 +104,7 @@ QDebug operator<<(QDebug s, Qt::TimerType t)
 }
 #endif
 
-static void calculateCoarseTimerTimeout(QTimerInfo *t, timespec currentTime)
+static void calculateCoarseTimerTimeout(QTimerInfo *t, timespec now)
 {
     // The coarse timer works like this:
     //  - interval under 40 ms: round to even
@@ -212,106 +122,114 @@ static void calculateCoarseTimerTimeout(QTimerInfo *t, timespec currentTime)
     //
     // The objective is to make most timers wake up at the same time, thereby reducing CPU wakeups.
 
-    uint interval = uint(t->interval);
-    uint msec = uint(t->timeout.tv_nsec) / 1000 / 1000;
-    Q_ASSERT(interval >= 20);
+    Q_ASSERT(t->interval >= 20ms);
+
+    auto recalculate = [&](const milliseconds fracMsec) {
+        if (fracMsec == 1000ms) {
+            ++t->timeout.tv_sec;
+            t->timeout.tv_nsec = 0;
+        } else {
+            t->timeout.tv_nsec = nanoseconds{fracMsec}.count();
+        }
+
+        if (t->timeout < now)
+            t->timeout += t->interval;
+    };
 
     // Calculate how much we can round and still keep within 5% error
-    uint absMaxRounding = interval / 20;
+    const milliseconds absMaxRounding = t->interval / 20;
 
-    if (interval < 100 && interval != 25 && interval != 50 && interval != 75) {
+    auto fracMsec = duration_cast<milliseconds>(nanoseconds{t->timeout.tv_nsec});
+
+    if (t->interval < 100ms && t->interval != 25ms && t->interval != 50ms && t->interval != 75ms) {
+        auto fracCount = fracMsec.count();
         // special mode for timers of less than 100 ms
-        if (interval < 50) {
+        if (t->interval < 50ms) {
             // round to even
             // round towards multiples of 50 ms
-            bool roundUp = (msec % 50) >= 25;
-            msec >>= 1;
-            msec |= uint(roundUp);
-            msec <<= 1;
+            bool roundUp = (fracCount % 50) >= 25;
+            fracCount >>= 1;
+            fracCount |= roundUp;
+            fracCount <<= 1;
         } else {
             // round to multiple of 4
             // round towards multiples of 100 ms
-            bool roundUp = (msec % 100) >= 50;
-            msec >>= 2;
-            msec |= uint(roundUp);
-            msec <<= 2;
+            bool roundUp = (fracCount % 100) >= 50;
+            fracCount >>= 2;
+            fracCount |= roundUp;
+            fracCount <<= 2;
         }
-    } else {
-        uint min = qMax<int>(0, msec - absMaxRounding);
-        uint max = qMin(1000u, msec + absMaxRounding);
+        fracMsec = milliseconds{fracCount};
+        recalculate(fracMsec);
+        return;
+    }
 
-        // find the boundary that we want, according to the rules above
-        // extra rules:
-        // 1) whatever the interval, we'll take any round-to-the-second timeout
-        if (min == 0) {
-            msec = 0;
-            goto recalculate;
-        } else if (max == 1000) {
-            msec = 1000;
-            goto recalculate;
-        }
+    milliseconds min = std::max(0ms, fracMsec - absMaxRounding);
+    milliseconds max = std::min(1000ms, fracMsec + absMaxRounding);
 
-        uint wantedBoundaryMultiple;
+    // find the boundary that we want, according to the rules above
+    // extra rules:
+    // 1) whatever the interval, we'll take any round-to-the-second timeout
+    if (min == 0ms) {
+        fracMsec = 0ms;
+        recalculate(fracMsec);
+        return;
+    } else if (max == 1000ms) {
+        fracMsec = 1000ms;
+        recalculate(fracMsec);
+        return;
+    }
 
-        // 2) if the interval is a multiple of 500 ms and > 5000 ms, we'll always round
-        //    towards a round-to-the-second
-        // 3) if the interval is a multiple of 500 ms, we'll round towards the nearest
-        //    multiple of 500 ms
-        if ((interval % 500) == 0) {
-            if (interval >= 5000) {
-                msec = msec >= 500 ? max : min;
-                goto recalculate;
-            } else {
-                wantedBoundaryMultiple = 500;
-            }
-        } else if ((interval % 50) == 0) {
-            // 4) same for multiples of 250, 200, 100, 50
-            uint mult50 = interval / 50;
-            if ((mult50 % 4) == 0) {
-                // multiple of 200
-                wantedBoundaryMultiple = 200;
-            } else if ((mult50 % 2) == 0) {
-                // multiple of 100
-                wantedBoundaryMultiple = 100;
-            } else if ((mult50 % 5) == 0) {
-                // multiple of 250
-                wantedBoundaryMultiple = 250;
-            } else {
-                // multiple of 50
-                wantedBoundaryMultiple = 50;
-            }
+    milliseconds wantedBoundaryMultiple{25};
+
+    // 2) if the interval is a multiple of 500 ms and > 5000 ms, we'll always round
+    //    towards a round-to-the-second
+    // 3) if the interval is a multiple of 500 ms, we'll round towards the nearest
+    //    multiple of 500 ms
+    if ((t->interval % 500) == 0ms) {
+        if (t->interval >= 5s) {
+            fracMsec = fracMsec >= 500ms ? max : min;
+            recalculate(fracMsec);
+            return;
         } else {
-            wantedBoundaryMultiple = 25;
+            wantedBoundaryMultiple = 500ms;
+        }
+    } else if ((t->interval % 50) == 0ms) {
+        // 4) same for multiples of 250, 200, 100, 50
+        milliseconds mult50 = t->interval / 50;
+        if ((mult50 % 4) == 0ms) {
+            // multiple of 200
+            wantedBoundaryMultiple = 200ms;
+        } else if ((mult50 % 2) == 0ms) {
+            // multiple of 100
+            wantedBoundaryMultiple = 100ms;
+        } else if ((mult50 % 5) == 0ms) {
+            // multiple of 250
+            wantedBoundaryMultiple = 250ms;
+        } else {
+            // multiple of 50
+            wantedBoundaryMultiple = 50ms;
         }
-
-        uint base = msec / wantedBoundaryMultiple * wantedBoundaryMultiple;
-        uint middlepoint = base + wantedBoundaryMultiple / 2;
-        if (msec < middlepoint)
-            msec = qMax(base, min);
-        else
-            msec = qMin(base + wantedBoundaryMultiple, max);
     }
 
-recalculate:
-    if (msec == 1000u) {
-        ++t->timeout.tv_sec;
-        t->timeout.tv_nsec = 0;
-    } else {
-        t->timeout.tv_nsec = msec * 1000 * 1000;
-    }
+    milliseconds base = (fracMsec / wantedBoundaryMultiple) * wantedBoundaryMultiple;
+    milliseconds middlepoint = base + wantedBoundaryMultiple / 2;
+    if (fracMsec < middlepoint)
+        fracMsec = qMax(base, min);
+    else
+        fracMsec = qMin(base + wantedBoundaryMultiple, max);
 
-    if (t->timeout < currentTime)
-        t->timeout += interval;
+    recalculate(fracMsec);
 }
 
-static void calculateNextTimeout(QTimerInfo *t, timespec currentTime)
+static void calculateNextTimeout(QTimerInfo *t, timespec now)
 {
     switch (t->timerType) {
     case Qt::PreciseTimer:
     case Qt::CoarseTimer:
         t->timeout += t->interval;
-        if (t->timeout < currentTime) {
-            t->timeout = currentTime;
+        if (t->timeout < now) {
+            t->timeout = now;
             t->timeout += t->interval;
         }
 #ifdef QTIMERINFO_DEBUG
@@ -322,14 +240,15 @@ static void calculateNextTimeout(QTimerInfo *t, timespec currentTime)
         }
 #endif
         if (t->timerType == Qt::CoarseTimer)
-            calculateCoarseTimerTimeout(t, currentTime);
+            calculateCoarseTimerTimeout(t, now);
         return;
 
     case Qt::VeryCoarseTimer:
-        // we don't need to take care of the microsecond component of t->interval
-        t->timeout.tv_sec += t->interval;
-        if (t->timeout.tv_sec <= currentTime.tv_sec)
-            t->timeout.tv_sec = currentTime.tv_sec + t->interval;
+        // t->interval already rounded to full seconds in registerTimer()
+        const auto secs = duration_cast<seconds>(t->interval).count();
+        t->timeout.tv_sec += secs;
+        if (t->timeout.tv_sec <= now.tv_sec)
+            t->timeout.tv_sec = now.tv_sec + secs;
 #ifdef QTIMERINFO_DEBUG
         t->expected.tv_sec += t->interval;
         if (t->expected.tv_sec <= currentTime.tv_sec)
@@ -352,65 +271,59 @@ static void calculateNextTimeout(QTimerInfo *t, timespec currentTime)
 */
 bool QTimerInfoList::timerWait(timespec &tm)
 {
-    timespec currentTime = updateCurrentTime();
-    repairTimersIfNeeded();
+    timespec now = updateCurrentTime();
 
+    auto isWaiting = [](QTimerInfo *tinfo) { return !tinfo->activateRef; };
     // Find first waiting timer not already active
-    QTimerInfo *t = nullptr;
-    for (QTimerInfoList::const_iterator it = constBegin(); it != constEnd(); ++it) {
-        if (!(*it)->activateRef) {
-            t = *it;
-            break;
-        }
-    }
+    auto it = std::find_if(cbegin(), cend(), isWaiting);
+    if (it == cend())
+        return false;
 
-    if (!t)
-      return false;
-
-    if (currentTime < t->timeout) {
-        // time to wait
-        tm = roundToMillisecond(t->timeout - currentTime);
-    } else {
-        // no time to wait
-        tm.tv_sec  = 0;
-        tm.tv_nsec = 0;
-    }
+    QTimerInfo *t = *it;
+    if (now < t->timeout) // Time to wait
+        tm = roundToMillisecond(t->timeout - now);
+    else // No time to wait
+        tm = {0, 0};
 
     return true;
 }
 
 /*
-  Returns the timer's remaining time in milliseconds with the given timerId, or
-  null if there is nothing left. If the timer id is not found in the list, the
-  returned value will be -1. If the timer is overdue, the returned value will be 0.
+  Returns the timer's remaining time in milliseconds with the given timerId.
+  If the timer id is not found in the list, the returned value will be -1.
+  If the timer is overdue, the returned value will be 0.
 */
-int QTimerInfoList::timerRemainingTime(int timerId)
+qint64 QTimerInfoList::timerRemainingTime(int timerId)
 {
-    timespec currentTime = updateCurrentTime();
-    repairTimersIfNeeded();
-    timespec tm = {0, 0};
+    return remainingDuration(timerId).count();
+}
 
-    for (int i = 0; i < size(); ++i) {
-        QTimerInfo *t = at(i);
-        if (t->id == timerId) {
-            if (currentTime < t->timeout) {
-                // time to wait
-                tm = roundToMillisecond(t->timeout - currentTime);
-                return tm.tv_sec*1000 + tm.tv_nsec/1000/1000;
-            } else {
-                return 0;
-            }
-        }
+milliseconds QTimerInfoList::remainingDuration(int timerId)
+{
+    timespec now = updateCurrentTime();
+
+    auto it = findTimerById(timerId);
+    if (it == cend()) {
+#ifndef QT_NO_DEBUG
+        qWarning("QTimerInfoList::timerRemainingTime: timer id %i not found", timerId);
+#endif
+        return milliseconds{-1};
     }
 
-#ifndef QT_NO_DEBUG
-    qWarning("QTimerInfoList::timerRemainingTime: timer id %i not found", timerId);
-#endif
-
-    return -1;
+    const QTimerInfo *t = *it;
+    if (now < t->timeout) // time to wait
+        return timespecToChronoMs(roundToMillisecond(t->timeout - now));
+    else
+        return milliseconds{0};
 }
 
 void QTimerInfoList::registerTimer(int timerId, qint64 interval, Qt::TimerType timerType, QObject *object)
+{
+    registerTimer(timerId, milliseconds{interval}, timerType, object);
+}
+
+void QTimerInfoList::registerTimer(int timerId, milliseconds interval,
+                                   Qt::TimerType timerType, QObject *object)
 {
     QTimerInfo *t = new QTimerInfo;
     t->id = timerId;
@@ -433,30 +346,27 @@ void QTimerInfoList::registerTimer(int timerId, qint64 interval, Qt::TimerType t
         // so our boundaries are 20 ms and 20 s
         // below 20 ms, 5% inaccuracy is below 1 ms, so we convert to high precision
         // above 20 s, 5% inaccuracy is above 1 s, so we convert to VeryCoarseTimer
-        if (interval >= 20000) {
+        if (interval >= 20s) {
             t->timerType = Qt::VeryCoarseTimer;
         } else {
             t->timeout = expected;
-            if (interval <= 20) {
+            if (interval <= 20ms) {
                 t->timerType = Qt::PreciseTimer;
                 // no adjustment is necessary
-            } else if (interval <= 20000) {
+            } else if (interval <= 20s) {
                 calculateCoarseTimerTimeout(t, currentTime);
             }
             break;
         }
         Q_FALLTHROUGH();
     case Qt::VeryCoarseTimer:
-        // the very coarse timer is based on full second precision,
-        // so we keep the interval in seconds (round to closest second)
-        t->interval /= 500;
-        t->interval += 1;
-        t->interval >>= 1;
-        t->timeout.tv_sec = currentTime.tv_sec + t->interval;
+        const seconds secs = roundToSecs(t->interval);
+        t->interval = secs;
+        t->timeout.tv_sec = currentTime.tv_sec + secs.count();
         t->timeout.tv_nsec = 0;
 
         // if we're past the half-second mark, increase the timeout again
-        if (currentTime.tv_nsec > 500*1000*1000)
+        if (currentTime.tv_nsec > nanoseconds{500ms}.count())
             ++t->timeout.tv_sec;
     }
 
@@ -474,22 +384,19 @@ void QTimerInfoList::registerTimer(int timerId, qint64 interval, Qt::TimerType t
 
 bool QTimerInfoList::unregisterTimer(int timerId)
 {
+    auto it = findTimerById(timerId);
+    if (it == cend())
+        return false; // id not found
+
     // set timer inactive
-    for (int i = 0; i < size(); ++i) {
-        QTimerInfo *t = at(i);
-        if (t->id == timerId) {
-            // found it
-            removeAt(i);
-            if (t == firstTimerInfo)
-                firstTimerInfo = nullptr;
-            if (t->activateRef)
-                *(t->activateRef) = nullptr;
-            delete t;
-            return true;
-        }
-    }
-    // id not found
-    return false;
+    QTimerInfo *t = *it;
+    if (t == firstTimerInfo)
+        firstTimerInfo = nullptr;
+    if (t->activateRef)
+        *(t->activateRef) = nullptr;
+    delete t;
+    erase(it);
+    return true;
 }
 
 bool QTimerInfoList::unregisterTimers(QObject *object)
@@ -516,15 +423,9 @@ bool QTimerInfoList::unregisterTimers(QObject *object)
 QList<QAbstractEventDispatcher::TimerInfo> QTimerInfoList::registeredTimers(QObject *object) const
 {
     QList<QAbstractEventDispatcher::TimerInfo> list;
-    for (int i = 0; i < size(); ++i) {
-        const QTimerInfo * const t = at(i);
-        if (t->obj == object) {
-            list << QAbstractEventDispatcher::TimerInfo(t->id,
-                                                        (t->timerType == Qt::VeryCoarseTimer
-                                                         ? t->interval * 1000
-                                                         : t->interval),
-                                                        t->timerType);
-        }
+    for (const QTimerInfo *const t : std::as_const(*this)) {
+        if (t->obj == object)
+            list.emplaceBack(t->id, t->interval.count(), t->timerType);
     }
     return list;
 }
@@ -537,28 +438,24 @@ int QTimerInfoList::activateTimers()
     if (qt_disable_lowpriority_timers || isEmpty())
         return 0; // nothing to do
 
-    int n_act = 0, maxCount = 0;
     firstTimerInfo = nullptr;
 
-    timespec currentTime = updateCurrentTime();
-    // qDebug() << "Thread" << QThread::currentThreadId() << "woken up at" << currentTime;
-    repairTimersIfNeeded();
-
-
+    timespec now = updateCurrentTime();
+    // qDebug() << "Thread" << QThread::currentThreadId() << "woken up at" << now;
     // Find out how many timer have expired
-    for (QTimerInfoList::const_iterator it = constBegin(); it != constEnd(); ++it) {
-        if (currentTime < (*it)->timeout)
-            break;
-        maxCount++;
-    }
+    auto stillActive = [&now](const QTimerInfo *t) { return now < t->timeout; };
+    // Find first one still active (list is sorted by timeout)
+    auto it = std::find_if(cbegin(), cend(), stillActive);
+    auto maxCount = it - cbegin();
 
+    int n_act = 0;
     //fire the timers.
     while (maxCount--) {
         if (isEmpty())
             break;
 
         QTimerInfo *currentTimerInfo = constFirst();
-        if (currentTime < currentTimerInfo->timeout)
+        if (now < currentTimerInfo->timeout)
             break; // no timer has expired
 
         if (!firstTimerInfo) {
@@ -595,11 +492,11 @@ int QTimerInfoList::activateTimers()
 #endif
 
         // determine next timeout time
-        calculateNextTimeout(currentTimerInfo, currentTime);
+        calculateNextTimeout(currentTimerInfo, now);
 
         // reinsert timer
         timerInsert(currentTimerInfo);
-        if (currentTimerInfo->interval > 0)
+        if (currentTimerInfo->interval > 0ms)
             n_act++;
 
         // Send event, but don't allow it to recurse:
