@@ -22,7 +22,7 @@
 #include "qendian.h"
 #include "qresource.h"
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_NACL) && !defined(Q_OS_INTEGRITY)
+#if defined(Q_OS_UNIX) && !defined(Q_OS_INTEGRITY)
 #  define QT_USE_MMAP
 #  include "private/qcore_unix_p.h"
 // for mmap
@@ -914,7 +914,7 @@ end:
     if (!tn)
         return QString();
     QString str(tn_length / 2, Qt::Uninitialized);
-    qFromBigEndian<ushort>(tn, str.size(), str.data());
+    qFromBigEndian<char16_t>(tn, str.size(), str.data());
     return str;
 }
 
