@@ -3,7 +3,7 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 //#define QNATIVESOCKETENGINE_DEBUG
-#include "qnativesocketengine_p.h"
+#include "qnativesocketengine_p_p.h"
 #include "private/qnet_unix_p.h"
 #include "qiodevice.h"
 #include "qhostaddress.h"
@@ -18,7 +18,11 @@
 #include <errno.h>
 #include <fcntl.h>
 #ifndef QT_NO_IPV6IFNAME
+#ifdef Q_OS_LINUX
+#include <linux/if.h>
+#else // Q_OS_LINUX
 #include <net/if.h>
+#endif // !Q_OS_LINUX
 #endif
 #ifdef QT_LINUXBASE
 #include <arpa/inet.h>
@@ -40,6 +44,9 @@
 #include <sys/socket.h>
 #include <netinet/sctp.h>
 #endif
+#ifdef Q_OS_BSD4
+#  include <net/if_dl.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -799,7 +806,7 @@ bool QNativeSocketEnginePrivate::nativeHasPendingDatagrams() const
     // Peek 1 bytes into the next message.
     ssize_t readBytes;
     char c;
-    EINTR_LOOP(readBytes, ::recv(socketDescriptor, &c, 1, MSG_PEEK));
+    QT_EINTR_LOOP(readBytes, ::recv(socketDescriptor, &c, 1, MSG_PEEK));
 
     // If there's no error, or if our buffer was too small, there must be a
     // pending datagram.
@@ -818,7 +825,7 @@ qint64 QNativeSocketEnginePrivate::nativePendingDatagramSize() const
 #ifdef Q_OS_LINUX
     // Linux can return the actual datagram size if we use MSG_TRUNC
     char c;
-    EINTR_LOOP(recvResult, ::recv(socketDescriptor, &c, 1, MSG_PEEK | MSG_TRUNC));
+    QT_EINTR_LOOP(recvResult, ::recv(socketDescriptor, &c, 1, MSG_PEEK | MSG_TRUNC));
 #elif defined(SO_NREAD)
     // macOS can return the actual datagram size if we use SO_NREAD
     int value;
