@@ -26,17 +26,11 @@
 #include <security.h>
 #include <schnlsp.h>
 
-#if NTDDI_VERSION >= NTDDI_WINBLUE && !defined(Q_CC_MINGW)
+#if NTDDI_VERSION >= NTDDI_WINBLUE && defined(SECBUFFER_APPLICATION_PROTOCOLS)
 // ALPN = Application Layer Protocol Negotiation
 #define SUPPORTS_ALPN 1
 #endif
 
-// Redstone 5/1809 has all the API available, but TLS 1.3 is not enabled until a later version of
-// Win 10, checked at runtime in supportsTls13()
-#if defined(NTDDI_WIN10_RS5) && NTDDI_VERSION >= NTDDI_WIN10_RS5
-#define SUPPORTS_TLS13 1
-#endif
-
 // Not defined in MinGW
 #ifndef SECBUFFER_ALERT
 #define SECBUFFER_ALERT 17
@@ -139,7 +133,6 @@ QList<QSslCipher> defaultCiphers()
 {
     // Previously the code was in QSslSocketBackendPrivate.
     QList<QSslCipher> ciphers;
-    // @temp (I hope), stolen from qsslsocket_winrt.cpp
     const QString protocolStrings[] = { QStringLiteral("TLSv1"), QStringLiteral("TLSv1.1"),
                                         QStringLiteral("TLSv1.2"), QStringLiteral("TLSv1.3") };
 QT_WARNING_PUSH
@@ -299,7 +292,11 @@ QList<QSslCertificate> QSchannelBackend::systemCaCertificatesImplementation()
     // Similar to non-Darwin version found in qtlsbackend_openssl.cpp,
     // QTlsPrivate::systemCaCertificates function.
     QList<QSslCertificate> systemCerts;
-    auto hSystemStore = QHCertStorePointer(CertOpenSystemStore(0, L"ROOT"));
+
+    auto hSystemStore = QHCertStorePointer(
+            CertOpenStore(CERT_STORE_PROV_SYSTEM, 0, 0,
+                          CERT_STORE_READONLY_FLAG | CERT_SYSTEM_STORE_CURRENT_USER, L"ROOT"));
+
     if (hSystemStore) {
         PCCERT_CONTEXT pc = nullptr;
         while ((pc = CertFindCertificateInStore(hSystemStore.get(), X509_ASN_ENCODING, 0,
@@ -320,6 +317,11 @@ QTlsPrivate::X509DerReaderPtr QSchannelBackend::X509DerReader() const
     return QTlsPrivate::X509CertificateGeneric::certificatesFromDer;
 }
 
+QTlsPrivate::X509Pkcs12ReaderPtr QSchannelBackend::X509Pkcs12Reader() const
+{
+    return QTlsPrivate::X509CertificateSchannel::importPkcs12;
+}
+
 namespace {
 
 SecBuffer createSecBuffer(void *ptr, unsigned long length, unsigned long bufferType)
@@ -384,7 +386,6 @@ QString schannelErrorToString(qint32 status)
 
 bool supportsTls13()
 {
-#ifdef SUPPORTS_TLS13
     static bool supported = []() {
         const auto current = QOperatingSystemVersion::current();
         // 20221 just happens to be the preview version I run on my laptop where I tested TLS 1.3.
@@ -392,10 +393,8 @@ bool supportsTls13()
                 QOperatingSystemVersion(QOperatingSystemVersion::Windows, 10, 0, 20221);
         return current >= minimum;
     }();
+
     return supported;
-#else
-    return false;
-#endif
 }
 
 DWORD toSchannelProtocol(QSsl::SslProtocol protocol)
@@ -460,7 +459,6 @@ QT_WARNING_POP
     return protocols;
 }
 
-#ifdef SUPPORTS_TLS13
 // In the new API that descended down upon us we are not asked which protocols we want
 // but rather which protocols we don't want. So now we have this function to disable
 // anything that is not enabled.
@@ -470,7 +468,6 @@ DWORD toSchannelProtocolNegated(QSsl::SslProtocol protocol)
     protocols &= ~toSchannelProtocol(protocol); // minus the one(s) we want
     return protocols;
 }
-#endif
 
 /*!
     \internal
@@ -680,6 +677,10 @@ qint64 checkIncompleteData(const SecBuffer &secBuffer)
     return 0;
 }
 
+DWORD defaultCredsFlag()
+{
+    return qEnvironmentVariableIsSet("QT_SCH_DEFAULT_CREDS") ? 0 : SCH_CRED_NO_DEFAULT_CREDS;
+}
 } // anonymous namespace
 
 
@@ -719,6 +720,8 @@ bool TlsCryptographSchannel::sendToken(void *token, unsigned long tokenLength, b
     Q_ASSERT(d);
     auto *plainSocket = d->plainTcpSocket();
     Q_ASSERT(plainSocket);
+    if (plainSocket->state() == QAbstractSocket::UnconnectedState || !plainSocket->isValid())
+        return false;
 
     const qint64 written = plainSocket->write(static_cast<const char *>(token), tokenLength);
     if (written != qint64(tokenLength)) {
@@ -835,8 +838,7 @@ bool TlsCryptographSchannel::acquireCredentialsHandle()
         certsCount = 1;
         Q_ASSERT(localCertContext);
     }
-    void *credentials = nullptr;
-#ifdef SUPPORTS_TLS13
+
     TLS_PARAMETERS tlsParameters = {
         0,
         nullptr,
@@ -845,70 +847,33 @@ bool TlsCryptographSchannel::acquireCredentialsHandle()
         nullptr,
         0
     };
-    if (supportsTls13()) {
-        SCH_CREDENTIALS *cred = new SCH_CREDENTIALS{
-            SCH_CREDENTIALS_VERSION,
-            0,
-            certsCount,
-            &localCertContext,
-            nullptr,
-            0,
-            nullptr,
-            0,
-            SCH_CRED_REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT
-                    | SCH_CRED_NO_DEFAULT_CREDS,
-            1,
-            &tlsParameters
-        };
-        credentials = cred;
-    } else
-#endif // SUPPORTS_TLS13
-    {
-        SCHANNEL_CRED *cred = new SCHANNEL_CRED{
-            SCHANNEL_CRED_VERSION, // dwVersion
-            certsCount, // cCreds
-            &localCertContext, // paCred (certificate(s) containing a private key for authentication)
-            nullptr, // hRootStore
 
-            0, // cMappers (reserved)
-            nullptr, // aphMappers (reserved)
-
-            0, // cSupportedAlgs
-            nullptr, // palgSupportedAlgs (nullptr = system default)
-
-            protocols, // grbitEnabledProtocols
-            0, // dwMinimumCipherStrength (0 = system default)
-            0, // dwMaximumCipherStrength (0 = system default)
-            0, // dwSessionLifespan (0 = schannel default, 10 hours)
-            SCH_CRED_REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT
-                    | SCH_CRED_NO_DEFAULT_CREDS, // dwFlags
-            0 // dwCredFormat (must be 0)
-        };
-        credentials = cred;
-    }
-    Q_ASSERT(credentials != nullptr);
+    SCH_CREDENTIALS credentials = {
+        SCH_CREDENTIALS_VERSION,
+        0,
+        certsCount,
+        &localCertContext,
+        nullptr,
+        0,
+        nullptr,
+        0,
+        SCH_CRED_REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT | defaultCredsFlag(),
+        1,
+        &tlsParameters
+    };
 
     TimeStamp expiration{};
     auto status = AcquireCredentialsHandle(nullptr, // pszPrincipal (unused)
                                            const_cast<wchar_t *>(UNISP_NAME), // pszPackage
                                            isClient ? SECPKG_CRED_OUTBOUND : SECPKG_CRED_INBOUND, // fCredentialUse
                                            nullptr, // pvLogonID (unused)
-                                           credentials, // pAuthData
+                                           &credentials, // pAuthData
                                            nullptr, // pGetKeyFn (unused)
                                            nullptr, // pvGetKeyArgument (unused)
                                            &credentialHandle, // phCredential
                                            &expiration // ptsExpir
     );
 
-#ifdef SUPPORTS_TLS13
-    if (supportsTls13()) {
-        delete static_cast<SCH_CREDENTIALS *>(credentials);
-    } else
-#endif // SUPPORTS_TLS13
-    {
-        delete static_cast<SCHANNEL_CRED *>(credentials);
-    }
-
     if (status != SEC_E_OK) {
         setErrorAndEmit(d, QAbstractSocket::SslInternalError, schannelErrorToString(status));
         return false;
@@ -1113,7 +1078,7 @@ bool TlsCryptographSchannel::performHandshake()
     auto *plainSocket = d->plainTcpSocket();
     Q_ASSERT(plainSocket);
 
-    if (plainSocket->state() == QAbstractSocket::UnconnectedState) {
+    if (plainSocket->state() == QAbstractSocket::UnconnectedState || !plainSocket->isValid()) {
         setErrorAndEmit(d, QAbstractSocket::RemoteHostClosedError,
                         QSslSocket::tr("The TLS/SSL connection has been closed"));
         return false;
@@ -1483,7 +1448,7 @@ void TlsCryptographSchannel::transmit()
         return; // This function should not have been called
 
     // Can happen if called through QSslSocket::abort->QSslSocket::close->QSslSocket::flush->here
-    if (plainSocket->state() == QAbstractSocket::SocketState::UnconnectedState)
+    if (plainSocket->state() == QAbstractSocket::UnconnectedState || !plainSocket->isValid())
         return;
 
     if (schannelState != SchannelState::Done) {
@@ -1556,133 +1521,132 @@ void TlsCryptographSchannel::transmit()
         }
     }
 
-    if (q->isEncrypted()) { // Decrypt data from remote
-        int totalRead = 0;
-        bool hadIncompleteData = false;
-        const auto readBufferMaxSize = d->maxReadBufferSize();
-        while (!readBufferMaxSize || buffer.size() < readBufferMaxSize) {
-            if (missingData > plainSocket->bytesAvailable()
-                && (!readBufferMaxSize || readBufferMaxSize >= missingData)) {
+    int totalRead = 0;
+    bool hadIncompleteData = false;
+    const auto readBufferMaxSize = d->maxReadBufferSize();
+    while (!readBufferMaxSize || buffer.size() < readBufferMaxSize) {
+        if (missingData > plainSocket->bytesAvailable()
+            && (!readBufferMaxSize || readBufferMaxSize >= missingData)) {
 #ifdef QSSLSOCKET_DEBUG
-                qCDebug(lcTlsBackendSchannel, "We're still missing %lld bytes, will check later.",
-                        missingData);
+            qCDebug(lcTlsBackendSchannel, "We're still missing %lld bytes, will check later.",
+                    missingData);
+#endif
+            break;
+        }
+
+        missingData = 0;
+        const qint64 bytesRead = readToBuffer(intermediateBuffer, plainSocket);
+#ifdef QSSLSOCKET_DEBUG
+        qCDebug(lcTlsBackendSchannel, "Read %lld encrypted bytes from the socket", bytesRead);
+#endif
+        if (intermediateBuffer.length() == 0 || (hadIncompleteData && bytesRead == 0)) {
+#ifdef QSSLSOCKET_DEBUG
+            qCDebug(lcTlsBackendSchannel,
+                    hadIncompleteData ? "No new data received, leaving loop!"
+                                      : "Nothing to decrypt, leaving loop!");
+#endif
+            break;
+        }
+        hadIncompleteData = false;
+#ifdef QSSLSOCKET_DEBUG
+        qCDebug(lcTlsBackendSchannel, "Total amount of bytes to decrypt: %d",
+                intermediateBuffer.length());
+#endif
+
+        SecBuffer dataBuffer[4]{
+            createSecBuffer(intermediateBuffer, SECBUFFER_DATA),
+            createSecBuffer(nullptr, 0, SECBUFFER_EMPTY),
+            createSecBuffer(nullptr, 0, SECBUFFER_EMPTY),
+            createSecBuffer(nullptr, 0, SECBUFFER_EMPTY)
+        };
+        SecBufferDesc message{
+            SECBUFFER_VERSION,
+            ARRAYSIZE(dataBuffer),
+            dataBuffer
+        };
+        auto status = DecryptMessage(&contextHandle, &message, 0, nullptr);
+        if (status == SEC_E_OK || status == SEC_I_RENEGOTIATE || status == SEC_I_CONTEXT_EXPIRED) {
+            // There can still be 0 output even if it succeeds, this is fine
+            if (dataBuffer[1].cbBuffer > 0) {
+                // It is always decrypted in-place.
+                // But [0] is the STREAM_HEADER, [1] is the DATA and [2] is the STREAM_TRAILER.
+                // The pointers in all of those still point into 'intermediateBuffer'.
+                buffer.append(static_cast<char *>(dataBuffer[1].pvBuffer),
+                                dataBuffer[1].cbBuffer);
+                totalRead += dataBuffer[1].cbBuffer;
+#ifdef QSSLSOCKET_DEBUG
+                qCDebug(lcTlsBackendSchannel, "Decrypted %lu bytes. New read buffer size: %d",
+                        dataBuffer[1].cbBuffer, buffer.size());
 #endif
-                break;
             }
-
-            missingData = 0;
-            const qint64 bytesRead = readToBuffer(intermediateBuffer, plainSocket);
-#ifdef QSSLSOCKET_DEBUG
-            qCDebug(lcTlsBackendSchannel, "Read %lld encrypted bytes from the socket", bytesRead);
-#endif
-            if (intermediateBuffer.length() == 0 || (hadIncompleteData && bytesRead == 0)) {
-#ifdef QSSLSOCKET_DEBUG
-                qCDebug(lcTlsBackendSchannel,
-                        (hadIncompleteData ? "No new data received, leaving loop!"
-                                           : "Nothing to decrypt, leaving loop!"));
-#endif
-                break;
-            }
-            hadIncompleteData = false;
-#ifdef QSSLSOCKET_DEBUG
-            qCDebug(lcTlsBackendSchannel, "Total amount of bytes to decrypt: %d",
-                    intermediateBuffer.length());
-#endif
-
-            SecBuffer dataBuffer[4]{
-                createSecBuffer(intermediateBuffer, SECBUFFER_DATA),
-                createSecBuffer(nullptr, 0, SECBUFFER_EMPTY),
-                createSecBuffer(nullptr, 0, SECBUFFER_EMPTY),
-                createSecBuffer(nullptr, 0, SECBUFFER_EMPTY)
-            };
-            SecBufferDesc message{
-                SECBUFFER_VERSION,
-                ARRAYSIZE(dataBuffer),
-                dataBuffer
-            };
-            auto status = DecryptMessage(&contextHandle, &message, 0, nullptr);
-            if (status == SEC_E_OK || status == SEC_I_RENEGOTIATE || status == SEC_I_CONTEXT_EXPIRED) {
-                // There can still be 0 output even if it succeeds, this is fine
-                if (dataBuffer[1].cbBuffer > 0) {
-                    // It is always decrypted in-place.
-                    // But [0] is the STREAM_HEADER, [1] is the DATA and [2] is the STREAM_TRAILER.
-                    // The pointers in all of those still point into 'intermediateBuffer'.
-                    buffer.append(static_cast<char *>(dataBuffer[1].pvBuffer),
-                                  dataBuffer[1].cbBuffer);
-                    totalRead += dataBuffer[1].cbBuffer;
-#ifdef QSSLSOCKET_DEBUG
-                    qCDebug(lcTlsBackendSchannel, "Decrypted %lu bytes. New read buffer size: %d",
-                            dataBuffer[1].cbBuffer, buffer.size());
-#endif
-                }
-                if (dataBuffer[3].BufferType == SECBUFFER_EXTRA) {
-                    // https://docs.microsoft.com/en-us/windows/desktop/secauthn/extra-buffers-returned-by-schannel
-                    // dataBuffer[3].cbBuffer indicates the amount of bytes _NOT_ processed,
-                    // the rest need to be stored.
-                    retainExtraData(intermediateBuffer, dataBuffer[3]);
-                } else {
-                    intermediateBuffer.resize(0);
-                }
-            }
-
-            if (status == SEC_E_INCOMPLETE_MESSAGE) {
-                missingData = checkIncompleteData(dataBuffer[0]);
-#ifdef QSSLSOCKET_DEBUG
-                qCDebug(lcTlsBackendSchannel,
-                        "We didn't have enough data to decrypt anything, will try again!");
-#endif
-                // We try again, but if we don't get any more data then we leave
-                hadIncompleteData = true;
-            } else if (status == SEC_E_INVALID_HANDLE) {
-                // I don't think this should happen, if it does we're done...
-                qCWarning(lcTlsBackendSchannel, "The internal SSPI handle is invalid!");
-                Q_UNREACHABLE();
-            } else if (status == SEC_E_INVALID_TOKEN) {
-                qCWarning(lcTlsBackendSchannel, "Got SEC_E_INVALID_TOKEN!");
-                Q_UNREACHABLE(); // Happened once due to a bug, but shouldn't generally happen(?)
-            } else if (status == SEC_E_MESSAGE_ALTERED) {
-                // The message has been altered, disconnect now.
-                shutdown = true; // skips sending the shutdown alert
-                disconnectFromHost();
-                setErrorAndEmit(d, QAbstractSocket::SslInternalError,
-                                schannelErrorToString(status));
-                break;
-            } else if (status == SEC_E_OUT_OF_SEQUENCE) {
-                // @todo: I don't know if this one is actually "fatal"..
-                // This path might never be hit as it seems this is for connection-oriented connections,
-                // while SEC_E_MESSAGE_ALTERED is for stream-oriented ones (what we use).
-                shutdown = true; // skips sending the shutdown alert
-                disconnectFromHost();
-                setErrorAndEmit(d, QAbstractSocket::SslInternalError,
-                                schannelErrorToString(status));
-                break;
-            } else if (status == SEC_I_CONTEXT_EXPIRED) {
-                // 'remote' has initiated a shutdown
-                disconnectFromHost();
-                setErrorAndEmit(d, QAbstractSocket::RemoteHostClosedError,
-                                schannelErrorToString(status));
-                break;
-            } else if (status == SEC_I_RENEGOTIATE) {
-                // 'remote' wants to renegotiate
-#ifdef QSSLSOCKET_DEBUG
-                qCDebug(lcTlsBackendSchannel, "The peer wants to renegotiate.");
-#endif
-                schannelState = SchannelState::Renegotiate;
-                renegotiating = true;
-
-                // We need to call 'continueHandshake' or else there's no guarantee it ever gets called
-                continueHandshake();
-                break;
+            if (dataBuffer[3].BufferType == SECBUFFER_EXTRA) {
+                // https://docs.microsoft.com/en-us/windows/desktop/secauthn/extra-buffers-returned-by-schannel
+                // dataBuffer[3].cbBuffer indicates the amount of bytes _NOT_ processed,
+                // the rest need to be stored.
+                retainExtraData(intermediateBuffer, dataBuffer[3]);
+            } else {
+                intermediateBuffer.resize(0);
             }
         }
 
-        if (totalRead) {
-            if (bool *readyReadEmittedPointer = d->readyReadPointer())
-                *readyReadEmittedPointer = true;
-            emit q->readyRead();
-            emit q->channelReadyRead(0);
+        if (status == SEC_E_INCOMPLETE_MESSAGE) {
+            missingData = checkIncompleteData(dataBuffer[0]);
+#ifdef QSSLSOCKET_DEBUG
+            qCDebug(lcTlsBackendSchannel, "We didn't have enough data to decrypt anything, will try again!");
+#endif
+            // We try again, but if we don't get any more data then we leave
+            hadIncompleteData = true;
+        } else if (status == SEC_E_INVALID_HANDLE) {
+            // I don't think this should happen, if it does we're done...
+            qCWarning(lcTlsBackendSchannel, "The internal SSPI handle is invalid!");
+            Q_UNREACHABLE();
+        } else if (status == SEC_E_INVALID_TOKEN) {
+            // Supposedly we have an invalid token, it's under-documented what
+            // this means, so to be safe we disconnect.
+            shutdown = true;
+            disconnectFromHost();
+            setErrorAndEmit(d, QAbstractSocket::SslInternalError, schannelErrorToString(status));
+            break;
+        } else if (status == SEC_E_MESSAGE_ALTERED) {
+            // The message has been altered, disconnect now.
+            shutdown = true; // skips sending the shutdown alert
+            disconnectFromHost();
+            setErrorAndEmit(d, QAbstractSocket::SslInternalError,
+                            schannelErrorToString(status));
+            break;
+        } else if (status == SEC_E_OUT_OF_SEQUENCE) {
+            // @todo: I don't know if this one is actually "fatal"..
+            // This path might never be hit as it seems this is for connection-oriented connections,
+            // while SEC_E_MESSAGE_ALTERED is for stream-oriented ones (what we use).
+            shutdown = true; // skips sending the shutdown alert
+            disconnectFromHost();
+            setErrorAndEmit(d, QAbstractSocket::SslInternalError,
+                            schannelErrorToString(status));
+            break;
+        } else if (status == SEC_I_CONTEXT_EXPIRED) {
+            // 'remote' has initiated a shutdown
+            disconnectFromHost();
+            break;
+        } else if (status == SEC_I_RENEGOTIATE) {
+            // 'remote' wants to renegotiate
+#ifdef QSSLSOCKET_DEBUG
+            qCDebug(lcTlsBackendSchannel, "The peer wants to renegotiate.");
+#endif
+            schannelState = SchannelState::Renegotiate;
+            renegotiating = true;
+
+            // We need to call 'continueHandshake' or else there's no guarantee it ever gets called
+            continueHandshake();
+            break;
         }
     }
+
+    if (totalRead) {
+        if (bool *readyReadEmittedPointer = d->readyReadPointer())
+            *readyReadEmittedPointer = true;
+        emit q->readyRead();
+        emit q->channelReadyRead(0);
+    }
 }
 
 void TlsCryptographSchannel::sendShutdown()
@@ -1776,30 +1740,36 @@ void TlsCryptographSchannel::disconnectFromHost()
     if (SecIsValidHandle(&contextHandle)) {
         if (!shutdown) {
             shutdown = true;
-            if (plainSocket->state() != QAbstractSocket::UnconnectedState) {
-                if (q->isEncrypted()) {
-                    // Read as much as possible because this is likely our last chance
-                    qint64 tempMax = d->maxReadBufferSize();
-                    d->setMaxReadBufferSize(0);
-                    transmit();
-                    d->setMaxReadBufferSize(tempMax);
-                    sendShutdown();
-                }
+            if (plainSocket->state() != QAbstractSocket::UnconnectedState && q->isEncrypted()) {
+                sendShutdown();
+                transmit();
             }
         }
     }
-    if (plainSocket->state() != QAbstractSocket::UnconnectedState)
-        plainSocket->disconnectFromHost();
+    plainSocket->disconnectFromHost();
 }
 
 void TlsCryptographSchannel::disconnected()
 {
     Q_ASSERT(d);
+    auto *plainSocket = d->plainTcpSocket();
+    Q_ASSERT(plainSocket);
+    d->setEncrypted(false);
 
     shutdown = true;
-    d->setEncrypted(false);
-    deallocateContext();
-    freeCredentialsHandle();
+    if (plainSocket->bytesAvailable() > 0 || hasUndecryptedData()) {
+        // Read as much as possible because this is likely our last chance
+        qint64 tempMax = d->maxReadBufferSize();
+        d->setMaxReadBufferSize(0); // Unlimited
+        transmit();
+        d->setMaxReadBufferSize(tempMax);
+        // Since there were bytes still available we don't want to deallocate
+        // our context yet. It will happen later, when the socket is re-used or
+        // destroyed.
+    } else {
+        deallocateContext();
+        freeCredentialsHandle();
+    }
 }
 
 QSslCipher TlsCryptographSchannel::sessionCipher() const
@@ -1992,7 +1962,10 @@ bool TlsCryptographSchannel::verifyCertContext(CERT_CONTEXT *certContext)
         // the Ca list, not just included during verification.
         // That being said, it's not trivial to add the root certificates (if and only if they
         // came from the system root store). And I don't see this mentioned in our documentation.
-        auto rootStore = QHCertStorePointer(CertOpenSystemStore(0, L"ROOT"));
+        auto rootStore = QHCertStorePointer(
+                CertOpenStore(CERT_STORE_PROV_SYSTEM, 0, 0,
+                              CERT_STORE_READONLY_FLAG | CERT_SYSTEM_STORE_CURRENT_USER, L"ROOT"));
+
         if (!rootStore) {
 #ifdef QSSLSOCKET_DEBUG
             qCWarning(lcTlsBackendSchannel, "Failed to open the system root CA certificate store!");
@@ -2106,10 +2079,40 @@ bool TlsCryptographSchannel::verifyCertContext(CERT_CONTEXT *certContext)
         verifyDepth = DWORD(q->peerVerifyDepth());
 
     const auto &caCertificates = q->sslConfiguration().caCertificates();
+
+    if (!rootCertOnDemandLoadingAllowed()
+            && !(chain->TrustStatus.dwErrorStatus & CERT_TRUST_IS_PARTIAL_CHAIN)
+            && (q->peerVerifyMode() == QSslSocket::VerifyPeer
+                    || (isClient && q->peerVerifyMode() == QSslSocket::AutoVerifyPeer))) {
+        // When verifying a peer Windows "helpfully" builds a chain that
+        // may include roots from the system store. But we don't want that if
+        // the user has set their own CA certificates.
+        // Since Windows claims this is not a partial chain the root is included
+        // and we have to check that it is one of our configured CAs.
+        CERT_CHAIN_ELEMENT *element = chain->rgpElement[chain->cElement - 1];
+        QSslCertificate certificate = getCertificateFromChainElement(element);
+        if (!caCertificates.contains(certificate)) {
+            auto error = QSslError(QSslError::CertificateUntrusted, certificate);
+            sslErrors += error;
+            emit q->peerVerifyError(error);
+            if (q->state() != QAbstractSocket::ConnectedState)
+                return false;
+        }
+    }
+
     QList<QSslCertificate> peerCertificateChain;
     for (DWORD i = 0; i < verifyDepth; i++) {
         CERT_CHAIN_ELEMENT *element = chain->rgpElement[i];
         QSslCertificate certificate = getCertificateFromChainElement(element);
+        if (certificate.isNull()) {
+            const auto &previousCert = !peerCertificateChain.isEmpty() ? peerCertificateChain.last()
+                                                                       : QSslCertificate();
+            auto error = QSslError(QSslError::SslError::UnableToGetIssuerCertificate, previousCert);
+            sslErrors += error;
+            emit q->peerVerifyError(error);
+            if (previousCert.isNull() || q->state() != QAbstractSocket::ConnectedState)
+                return false;
+        }
         const QList<QSslCertificateExtension> extensions = certificate.extensions();
 
 #ifdef QSSLSOCKET_DEBUG
