@@ -14,7 +14,7 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace {
+namespace QtPrivate {
 class PageItem : public QGraphicsItem
 {
 public:
@@ -25,7 +25,6 @@ public:
         qreal border = qMax(paperSize.height(), paperSize.width()) / 25;
         brect = QRectF(QPointF(-border, -border),
                        QSizeF(paperSize)+QSizeF(2*border, 2*border));
-        setCacheMode(DeviceCoordinateCache);
     }
 
     QRectF boundingRect() const override
@@ -136,7 +135,10 @@ protected:
     }
 };
 
-} // anonymous namespace
+} // namespace QtPrivate
+
+using GraphicsView = QtPrivate::GraphicsView;
+using PageItem = QtPrivate::PageItem;
 
 class QPrintPreviewWidgetPrivate : public QWidgetPrivate
 {
