@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qdrawutil.h"
 #include "qbitmap.h"
@@ -80,6 +44,7 @@ private:
 
 /*!
     \headerfile <qdrawutil.h>
+    \inmodule QtWidgets
     \title Drawing Utility Functions
 
     \sa QPainter
@@ -126,6 +91,20 @@ void qDrawShadeLine(QPainter *p, int x1, int y1, int x2, int y2,
         qWarning("qDrawShadeLine: Invalid parameters");
         return;
     }
+    PainterStateGuard painterGuard(p);
+    const qreal devicePixelRatio = p->device()->devicePixelRatio();
+    if (!qFuzzyCompare(devicePixelRatio, qreal(1))) {
+        painterGuard.save();
+        const qreal inverseScale = qreal(1) / devicePixelRatio;
+        p->scale(inverseScale, inverseScale);
+        x1 = qRound(devicePixelRatio * x1);
+        y1 = qRound(devicePixelRatio * y1);
+        x2 = qRound(devicePixelRatio * x2);
+        y2 = qRound(devicePixelRatio * y2);
+        lineWidth = qRound(devicePixelRatio * lineWidth);
+        midLineWidth = qRound(devicePixelRatio * midLineWidth);
+        p->translate(0.5, 0.5);
+    }
     int tlw = lineWidth*2 + midLineWidth;        // total line width
     QPen oldPen = p->pen();                        // save pen
     if (sunken)
@@ -251,10 +230,11 @@ void qDrawShadeRect(QPainter *p, int x, int y, int w, int h,
         p->scale(inverseScale, inverseScale);
         x = qRound(devicePixelRatio * x);
         y = qRound(devicePixelRatio * y);
-        w = qRound(devicePixelRatio * w);
-        h = qRound(devicePixelRatio * h);
+        w = devicePixelRatio * w;
+        h = devicePixelRatio * h;
         lineWidth = qRound(devicePixelRatio * lineWidth);
         midLineWidth = qRound(devicePixelRatio * midLineWidth);
+        p->translate(0.5, 0.5);
     }
 
     QPen oldPen = p->pen();
@@ -359,15 +339,18 @@ void qDrawShadePanel(QPainter *p, int x, int y, int w, int h,
 
     PainterStateGuard painterGuard(p);
     const qreal devicePixelRatio = p->device()->devicePixelRatio();
+    bool isTranslated = false;
     if (!qFuzzyCompare(devicePixelRatio, qreal(1))) {
         painterGuard.save();
         const qreal inverseScale = qreal(1) / devicePixelRatio;
         p->scale(inverseScale, inverseScale);
         x = qRound(devicePixelRatio * x);
         y = qRound(devicePixelRatio * y);
-        w = qRound(devicePixelRatio * w);
-        h = qRound(devicePixelRatio * h);
+        w = devicePixelRatio * w;
+        h = devicePixelRatio * h;
         lineWidth = qRound(devicePixelRatio * lineWidth);
+        p->translate(0.5, 0.5);
+        isTranslated = true;
     }
 
     QColor shade = pal.dark().color();
@@ -418,8 +401,11 @@ void qDrawShadePanel(QPainter *p, int x, int y, int w, int h,
         lines << QLineF(x1--, y1++, x2--, y2);
     }
     p->drawLines(lines);
-    if (fill)                                // fill with fill color
+    if (fill) {                                // fill with fill color
+        if (isTranslated)
+            p->translate(-0.5, -0.5);
         p->fillRect(x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2, *fill);
+    }
     p->setPen(oldPen);                        // restore pen
 }
 
@@ -450,14 +436,17 @@ static void qDrawWinShades(QPainter *p,
 
     PainterStateGuard painterGuard(p);
     const qreal devicePixelRatio = p->device()->devicePixelRatio();
+    bool isTranslated = false;
     if (!qFuzzyCompare(devicePixelRatio, qreal(1))) {
         painterGuard.save();
         const qreal inverseScale = qreal(1) / devicePixelRatio;
         p->scale(inverseScale, inverseScale);
         x = qRound(devicePixelRatio * x);
         y = qRound(devicePixelRatio * y);
-        w = qRound(devicePixelRatio * w);
-        h = qRound(devicePixelRatio * h);
+        w = devicePixelRatio * w;
+        h = devicePixelRatio * h;
+        p->translate(0.5, 0.5);
+        isTranslated = true;
     }
 
     QPen oldPen = p->pen();
@@ -474,8 +463,11 @@ static void qDrawWinShades(QPainter *p,
         QPoint d[3] = { QPoint(x+1, y+h-2), QPoint(x+w-2, y+h-2), QPoint(x+w-2, y+1) };
         p->setPen(c4);
         p->drawPolyline(d, 3);
-        if (fill)
+        if (fill) {
+            if (isTranslated)
+                p->translate(-0.5, -0.5);
             p->fillRect(QRect(x+2, y+2, w-4, h-4), *fill);
+        }
     }
     p->setPen(oldPen);
 }
@@ -598,9 +590,10 @@ void qDrawPlainRect(QPainter *p, int x, int y, int w, int h, const QColor &c,
         p->scale(inverseScale, inverseScale);
         x = qRound(devicePixelRatio * x);
         y = qRound(devicePixelRatio * y);
-        w = qRound(devicePixelRatio * w);
-        h = qRound(devicePixelRatio * h);
+        w = devicePixelRatio * w;
+        h = devicePixelRatio * h;
         lineWidth = qRound(devicePixelRatio * lineWidth);
+        p->translate(0.5, 0.5);
     }
 
     QPen   oldPen   = p->pen();
@@ -770,6 +763,7 @@ void qDrawWinButton(QPainter *p, const QRect &r,
 /*!
     \fn void qDrawWinPanel(QPainter *painter, const QRect &rect, const QPalette &palette,
              bool sunken, const QBrush *fill)
+    \relates <qdrawutil.h>
     \overload
 
     Draws the Windows-style panel at the rectangle specified by \a rect using
@@ -866,6 +860,7 @@ void qDrawPlainRect(QPainter *p, const QRect &r, const QColor &c,
 typedef QVarLengthArray<QPainter::PixmapFragment, 16> QPixmapFragmentsArray;
 
 /*!
+    \relates <qdrawutil.h>
     \since 4.6
 
     Draws the indicated \a sourceRect rectangle from the given \a pixmap into
@@ -883,7 +878,7 @@ typedef QVarLengthArray<QPainter::PixmapFragment, 16> QPixmapFragmentsArray;
 void qDrawBorderPixmap(QPainter *painter, const QRect &targetRect, const QMargins &targetMargins,
                        const QPixmap &pixmap, const QRect &sourceRect,const QMargins &sourceMargins,
                        const QTileRules &rules
-#ifndef Q_CLANG_QDOC
+#ifndef Q_QDOC
                        , QDrawBorderPixmap::DrawingHints hints
 #endif
                        )
