@@ -4,8 +4,8 @@
 #ifndef QMOCKPLUGIN_H
 #define QMOCKPLUGIN_H
 
-#include <QString>
-#include <QtPlugin>
+#include <QtCore/QString>
+#include <QtCore/QtPlugin>
 
 QT_BEGIN_NAMESPACE
 
