@@ -3,8 +3,10 @@
 
 #include <QTest>
 
+#include <qdebug.h>
 #include <qhash.h>
 #include <qmap.h>
+#include <qscopeguard.h>
 #include <qset.h>
 
 #include <algorithm>
@@ -42,6 +44,7 @@ private slots:
     void qmultihash_qhash_rvalue_ref_ctor();
     void qmultihash_qhash_rvalue_ref_unite();
     void qmultihashUnite();
+    void qmultihashSize();
 
     void compare();
     void compare2();
@@ -61,6 +64,7 @@ private slots:
     void eraseValidIteratorOnSharedHash();
     void equal_range();
     void insert_hash();
+    void multiHashStoresInReverseInsertionOrder();
 
     void emplace();
 
@@ -76,6 +80,8 @@ private slots:
 
     void reserveShared();
     void reserveLessThanCurrentAmount();
+    void reserveKeepCapacity_data();
+    void reserveKeepCapacity();
 
     void QTBUG98265();
 
@@ -904,16 +910,31 @@ class QGlobalQHashSeedResetter
     int oldSeed;
 public:
     // not entirely correct (may lost changes made by another thread between the query
-    // of the old and the setting of the new seed), but qSetGlobalQHashSeed doesn't
+    // of the old and the setting of the new seed), but setHashSeed() can't
     // return the old value, so this is the best we can do:
     explicit QGlobalQHashSeedResetter(int newSeed)
-        : oldSeed(qGlobalQHashSeed())
+        : oldSeed(getHashSeed())
     {
-        qSetGlobalQHashSeed(newSeed);
+        setHashSeed(newSeed);
     }
     ~QGlobalQHashSeedResetter()
     {
-        qSetGlobalQHashSeed(oldSeed);
+        setHashSeed(oldSeed);
+    }
+
+private:
+    // The functions are implemented to replace the deprecated
+    // qGlobalQHashSeed() and qSetGlobalQHashSeed()
+    static int getHashSeed()
+    {
+        return int(QHashSeed::globalSeed() & INT_MAX);
+    }
+    static void setHashSeed(int seed)
+    {
+        if (seed == 0)
+            QHashSeed::setDeterministicGlobalSeed();
+        else
+            QHashSeed::resetRandomGlobalSeed();
     }
 };
 
@@ -2034,6 +2055,78 @@ void tst_QHash::qmultihashUnite()
     }
 }
 
+void tst_QHash::qmultihashSize()
+{
+    // QMultiHash has an extra m_size member that counts the number of values,
+    // while d->size (shared with QHash) counts the number of distinct keys.
+    {
+        QMultiHash<int, int> hash;
+        QCOMPARE(hash.size(), 0);
+        QVERIFY(hash.isEmpty());
+
+        hash.insert(0, 42);
+        QCOMPARE(hash.size(), 1);
+        QVERIFY(!hash.isEmpty());
+
+        hash.insert(0, 42);
+        QCOMPARE(hash.size(), 2);
+        QVERIFY(!hash.isEmpty());
+
+        hash.emplace(0, 42);
+        QCOMPARE(hash.size(), 3);
+        QVERIFY(!hash.isEmpty());
+
+        QCOMPARE(hash.take(0), 42);
+        QCOMPARE(hash.size(), 2);
+        QVERIFY(!hash.isEmpty());
+
+        QCOMPARE(hash.remove(0), 2);
+        QCOMPARE(hash.size(), 0);
+        QVERIFY(hash.isEmpty());
+    }
+
+    {
+        QMultiHash<int, int> hash;
+        hash.emplace(0, 0);
+        hash.emplace(0, 0);
+        QCOMPARE(hash.size(), 2);
+        QVERIFY(!hash.isEmpty());
+
+        hash.emplace(0, 1);
+        QCOMPARE(hash.size(), 3);
+        QVERIFY(!hash.isEmpty());
+
+        QCOMPARE(hash.remove(0, 0), 2);
+        QCOMPARE(hash.size(), 1);
+        QVERIFY(!hash.isEmpty());
+
+        hash.remove(0);
+        QCOMPARE(hash.size(), 0);
+        QVERIFY(hash.isEmpty());
+    }
+
+    {
+        QMultiHash<int, int> hash;
+
+        hash[0] = 0;
+        QCOMPARE(hash.size(), 1);
+        QVERIFY(!hash.isEmpty());
+
+        hash.replace(0, 1);
+        QCOMPARE(hash.size(), 1);
+        QVERIFY(!hash.isEmpty());
+
+        hash.insert(0, 1);
+        hash.erase(hash.cbegin());
+        QCOMPARE(hash.size(), 1);
+        QVERIFY(!hash.isEmpty());
+
+        hash.erase(hash.cbegin());
+        QCOMPARE(hash.size(), 0);
+        QVERIFY(hash.isEmpty());
+    }
+}
+
 void tst_QHash::keys_values_uniqueKeys()
 {
     QMultiHash<QString, int> hash;
@@ -2394,6 +2487,24 @@ void tst_QHash::insert_hash()
     }
 }
 
+void tst_QHash::multiHashStoresInReverseInsertionOrder()
+{
+    const QString strings[] = {
+        u"zero"_s,
+        u"null"_s,
+        u"nada"_s,
+    };
+    {
+        QMultiHash<int, QString> hash;
+        for (const QString &string : strings)
+            hash.insert(0, string);
+        auto printOnFailure = qScopeGuard([&] { qDebug() << hash; });
+        QVERIFY(std::equal(hash.begin(), hash.end(),
+                           std::rbegin(strings), std::rend(strings)));
+        printOnFailure.dismiss();
+    }
+}
+
 void tst_QHash::emplace()
 {
     {
@@ -2678,6 +2789,40 @@ void tst_QHash::reserveLessThanCurrentAmount()
     }
 }
 
+void tst_QHash::reserveKeepCapacity_data()
+{
+    QTest::addColumn<qsizetype>("requested");
+    auto addRow = [](qsizetype requested) {
+        QTest::addRow("%td", ptrdiff_t(requested)) << requested;
+    };
+
+    QHash<int, int> testHash = {{1, 1}};
+    qsizetype minCapacity = testHash.capacity();
+    addRow(minCapacity - 1);
+    addRow(minCapacity + 0);
+    addRow(minCapacity + 1);
+    addRow(2 * minCapacity - 1);
+    addRow(2 * minCapacity + 0);
+    addRow(2 * minCapacity + 1);
+}
+
+void tst_QHash::reserveKeepCapacity()
+{
+    QFETCH(qsizetype, requested);
+
+    QHash<qsizetype, qsizetype> hash;
+    hash.reserve(requested);
+    qsizetype initialCapacity = hash.capacity();
+    QCOMPARE_GE(initialCapacity, requested);
+
+    // insert this many elements into the hash
+    for (qsizetype i = 0; i < requested; ++i)
+        hash.insert(i, i);
+
+    // it mustn't have increased capacity after inserting the elements
+    QCOMPARE(hash.capacity(), initialCapacity);
+}
+
 void tst_QHash::QTBUG98265()
 {
     QMultiHash<QUuid, QByteArray> a;
