@@ -27,6 +27,10 @@ private slots:
 
 void tst_QVulkan::vulkanInstance()
 {
+#ifdef Q_OS_ANDROID
+    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
+        QSKIP("Fails on Android 12 (QTBUG-111236)");
+#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -63,6 +67,10 @@ void tst_QVulkan::vulkanInstance()
 
 void tst_QVulkan::vulkanCheckSupported()
 {
+#ifdef Q_OS_ANDROID
+    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
+        QSKIP("Fails on Android 12 (QTBUG-111236)");
+#endif
     // Test the early calls to supportedLayers/extensions/apiVersion that need
     // the library and some basics, but do not initialize the instance.
     QVulkanInstance inst;
@@ -166,6 +174,10 @@ void tst_QVulkan::vulkan11()
 
 void tst_QVulkan::vulkanPlainWindow()
 {
+#ifdef Q_OS_ANDROID
+    QSKIP("Fails on Android 7 emulator (QTBUG-108328)");
+#endif
+
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -202,6 +214,10 @@ void tst_QVulkan::vulkanPlainWindow()
 
 void tst_QVulkan::vulkanVersionRequest()
 {
+#ifdef Q_OS_ANDROID
+    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
+        QSKIP("Fails on Android 12 (QTBUG-111236)");
+#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -248,6 +264,10 @@ static void waitForUnexposed(QWindow *w)
 
 void tst_QVulkan::vulkanWindow()
 {
+#ifdef Q_OS_ANDROID
+    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
+        QSKIP("Fails on Android 12 (QTBUG-111236)");
+#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
