@@ -17,7 +17,6 @@ public:
         m_lastQueries(Qt::ImhNone),
         m_action(QInputMethod::Click),
         m_cursorPosition(0),
-        m_lastEventType(QEvent::None),
         m_setFocusObjectCallCount(0)
     {}
 
@@ -51,10 +50,6 @@ public:
         m_action = action;
         m_cursorPosition = cursorPosition;
     }
-    virtual bool filterEvent(const QEvent *event) override
-    {
-        m_lastEventType = event->type(); return false;
-    }
     virtual void showInputPanel() override
     {
         m_visible = true;
@@ -94,7 +89,6 @@ public:
     Qt::InputMethodQueries m_lastQueries;
     QInputMethod::Action m_action;
     int m_cursorPosition;
-    int m_lastEventType;
     QRectF m_keyboardRect;
     int m_setFocusObjectCallCount;
 };
