@@ -8,8 +8,11 @@
 #include <QSignalSpy>
 #include <QStyledItemDelegate>
 #include <QTest>
+#include <QLabel>
 #include <private/qlistwidget_p.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+
 using IntList = QList<int>;
 
 class tst_QListWidget : public QObject
@@ -72,6 +75,8 @@ private slots:
     void sortItems();
     void sortHiddenItems();
     void sortHiddenItems_data();
+    void sortCheckStability_data();
+    void sortCheckStability();
     void closeEditor();
     void setData_data();
     void setData();
@@ -103,6 +108,7 @@ private slots:
     void moveRows();
     void moveRowsInvalid_data();
     void moveRowsInvalid();
+    void noopDragDrop();
 
 protected slots:
     void rowsAboutToBeInserted(const QModelIndex &parent, int first, int last)
@@ -1127,6 +1133,64 @@ void tst_QListWidget::sortHiddenItems()
     delete tw;
 }
 
+void tst_QListWidget::sortCheckStability_data() {
+    QTest::addColumn<Qt::SortOrder>("order");
+    QTest::addColumn<QVariantList>("initialList");
+    QTest::addColumn<QVariantList>("expectedList");
+
+    QTest::newRow("ascending strings")
+            << Qt::AscendingOrder
+            << QVariantList{ QString("a"), QString("b"), QString("b"), QString("a")}
+            << QVariantList{ QString("a"), QString("a"), QString("b"), QString("b")};
+
+    QTest::newRow("descending strings")
+            << Qt::DescendingOrder
+            << QVariantList{ QString("a"), QString("b"), QString("b"), QString("a")}
+            << QVariantList{ QString("b"), QString("b"), QString("a"), QString("a")};
+
+    QTest::newRow("ascending numbers")
+            << Qt::AscendingOrder
+            << QVariantList{ 1, 2, 2, 1}
+            << QVariantList{ 1, 1, 2, 2};
+
+    QTest::newRow("descending numbers")
+            << Qt::DescendingOrder
+            << QVariantList{ 1, 2, 2, 1}
+            << QVariantList{ 2, 2, 1, 1};
+}
+
+void tst_QListWidget::sortCheckStability() {
+    QFETCH(Qt::SortOrder, order);
+    QFETCH(const QVariantList, initialList);
+    QFETCH(const QVariantList, expectedList);
+
+    for (const QVariant &data : initialList) {
+        QListWidgetItem *item = new QListWidgetItem(testWidget);
+        item->setData(Qt::DisplayRole, data);
+    }
+
+    QAbstractItemModel *model = testWidget->model();
+    QList<QPersistentModelIndex> persistent;
+    for (int j = 0; j < model->rowCount(QModelIndex()); ++j)
+        persistent << model->index(j, 0, QModelIndex());
+
+    testWidget->sortItems(order);
+
+    QCOMPARE(testWidget->count(), expectedList.size());
+    for (int i = 0; i < testWidget->count(); ++i)
+        QCOMPARE(testWidget->item(i)->text(), expectedList.at(i).toString());
+
+    QVector<QListWidgetItem*> itemOrder(testWidget->count());
+    for (int i = 0; i < testWidget->count(); ++i)
+        itemOrder[i] = testWidget->item(i);
+
+    qobject_cast<QListModel*>(testWidget->model())->ensureSorted(0, order, 1, 1);
+    testWidget->sortItems(order);
+
+    for (int i = 0; i < testWidget->count(); ++i)
+        QCOMPARE(itemOrder[i],testWidget->item(i));
+}
+
 class TestListWidget : public QListWidget
 {
     Q_OBJECT
@@ -1746,7 +1810,7 @@ void tst_QListWidget::QTBUG14363_completerWithAnyKeyPressedEditTriggers()
     new QListWidgetItem(QLatin1String("completer"), &listWidget);
     listWidget.show();
     listWidget.setCurrentItem(item);
-    QApplication::setActiveWindow(&listWidget);
+    QApplicationPrivate::setActiveWindow(&listWidget);
     QVERIFY(QTest::qWaitForWindowActive(&listWidget));
     listWidget.setFocus();
     QCOMPARE(QApplication::focusWidget(), &listWidget);
@@ -1887,6 +1951,58 @@ void tst_QListWidget::createPersistentOnLayoutAboutToBeChangedAutoSort() // QTBU
     QCOMPARE(layoutChangedSpy.size(), 1);
 }
 
+// Test that dropping an item on or beneath itself remains a no-op
+void tst_QListWidget::noopDragDrop() // QTBUG-100128
+{
+    QListWidget listWidget;
+    QList<QListWidgetItem *> items;
+    for (int i = 0; i < 5; ++i) {
+        const QString number = QString::number(i);
+        QListWidgetItem *item = new QListWidgetItem(&listWidget);
+        item->setData(Qt::UserRole, number);
+        QLabel *label = new QLabel(number);
+        listWidget.setItemWidget(item, label);
+        items.append(item);
+    }
+
+    listWidget.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&listWidget));
+
+    const QRect &lastItemRect = listWidget.visualItemRect(items.at(4));
+    const QPoint &dragStart = lastItemRect.center();
+    const QPoint &dropPointNirvana = lastItemRect.center() + QPoint(20, 2 * lastItemRect.height());
+
+    // Implement check as a macro (not a method) to safely determine the error location.
+    // The macro checks that item data and item widget remain unchanged when drag&drop are executed.
+    // In order to verify that the assets do *not* change, we can't use QTRY*: These macros would
+    // spin the event loop only once, while 3/4 mouse events need to get processed.
+    // That's why we spin the event loop 13 times, to make sure other unexpected or pending events
+    // get processed.
+#define CHECK_ITEM {\
+        const QString number = QString::number(4);\
+        for (int i = 0; i < 13; ++i)\
+            QApplication::processEvents();\
+        QLabel *label = qobject_cast<QLabel *>(listWidget.itemWidget(items.at(4)));\
+        QVERIFY(label);\
+        QCOMPARE(label->text(), number);\
+        const QString &data = items.at(4)->data(Qt::UserRole).toString();\
+        QCOMPARE(data, number);\
+    }
+
+    // Test dropping last item beneath itself
+    QTest::mousePress(&listWidget, Qt::LeftButton, Qt::KeyboardModifiers(), dragStart);
+    QTest::mouseMove(&listWidget, dropPointNirvana);
+    QTest::mouseRelease(&listWidget, Qt::LeftButton);
+    CHECK_ITEM;
+
+    // Test dropping last item on itself
+    QTest::mousePress(&listWidget, Qt::LeftButton, Qt::KeyboardModifiers(), dragStart);
+    QTest::mouseMove(&listWidget, dropPointNirvana);
+    QTest::mouseMove(&listWidget, dragStart);
+    QTest::mouseRelease(&listWidget, Qt::LeftButton);
+    CHECK_ITEM;
+}
+
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
 void tst_QListWidget::clearItemData()
 {
