@@ -28,6 +28,8 @@
 
 #include <QtTest/private/qtesthelpers_p.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+
 using namespace QTestPrivate;
 
 // Compare a window position that may go through scaling in the platform plugin with fuzz.
@@ -51,6 +53,7 @@ public:
     tst_QWidget_window();
 
 public slots:
+    void init();
     void initTestCase();
     void cleanupTestCase();
     void cleanup();
@@ -110,6 +113,8 @@ private slots:
 
     void resetFocusObjectOnDestruction();
 
+    void cleanupOnDestruction();
+
 private:
     QSize m_testWidgetSize;
     const int m_fuzz;
@@ -127,6 +132,11 @@ void tst_QWidget_window::initTestCase()
 {
 }
 
+void tst_QWidget_window::init()
+{
+    QTest::failOnWarning(QRegularExpression(".*No such slot.*"));
+}
+
 void tst_QWidget_window::cleanupTestCase()
 {
 }
@@ -678,7 +688,7 @@ void tst_QWidget_window::tst_dnd()
 
     dndTestWidget.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dndTestWidget));
-    qApp->setActiveWindow(&dndTestWidget);
+    QApplicationPrivate::setActiveWindow(&dndTestWidget);
     QVERIFY(QTest::qWaitForWindowActive(&dndTestWidget));
 
     QMimeData mimeData;
@@ -1664,5 +1674,30 @@ void tst_QWidget_window::resetFocusObjectOnDestruction()
     QCOMPARE(focusObjectChangedSpy.last().last().value<QObject*>(), nullptr);
 }
 
+class CreateDestroyWidget : public QWidget
+{
+public:
+    using QWidget::create;
+    using QWidget::destroy;
+};
+
+void tst_QWidget_window::cleanupOnDestruction()
+{
+    CreateDestroyWidget widget;
+    QWidget child(&widget);
+
+    QWidget grandChild(&child);
+    // Ensure there's not a 1:1 native window hierarhcy that we could
+    // recurse during QWidget::destroy(), triggering the issue that
+    // we were failing to clean up when not destroyed via QWidget.
+    grandChild.setAttribute(Qt::WA_DontCreateNativeAncestors);
+    grandChild.winId();
+
+    widget.destroy();
+    widget.create();
+
+    widget.show();
+}
+
 QTEST_MAIN(tst_QWidget_window)
 #include "tst_qwidget_window.moc"
